@@ -1,33 +1,39 @@
 import datetime as dt
 import logging
+import os.path
 import pathlib
 import traceback
 from functools import wraps
-from typing import Any, Callable, List, Optional
+from typing import Any, Callable, List, Optional, Tuple
 
 import click
 from dateutil.parser import parse
 
 from metricflow.cli.cli_context import CLIContext
+import metricflow.cli.custom_click_types as click_custom
 from metricflow.configuration.config_builder import ConfigKey
 from metricflow.configuration.constants import (
+    CONFIG_DWH_CREDS_PATH,
     CONFIG_DWH_DB,
     CONFIG_DWH_DIALECT,
     CONFIG_DWH_HOST,
     CONFIG_DWH_PASSWORD,
     CONFIG_DWH_PORT,
+    CONFIG_DWH_PROJECT_ID,
     CONFIG_DWH_SCHEMA,
     CONFIG_DWH_USER,
     CONFIG_DWH_WAREHOUSE,
     CONFIG_EMAIL,
     CONFIG_MODEL_PATH,
+    CONFIG_DWH_HTTP_PATH,
+    CONFIG_DWH_ACCESS_TOKEN,
 )
-from metricflow.model.validations.validator_helpers import ValidationIssueLevel
+from metricflow.sql_clients.common_client import SqlDialect
 
 logger = logging.getLogger(__name__)
 
 # MetricFlow config keys
-MF_CONFIG_KEYS = [
+MF_CONFIG_KEYS = (
     ConfigKey(key=CONFIG_EMAIL, comment="Optional"),
     ConfigKey(
         key=CONFIG_MODEL_PATH,
@@ -35,38 +41,74 @@ MF_CONFIG_KEYS = [
         comment="Path to directory containing defined models (Leave until after DWH setup)",
     ),
     ConfigKey(key=CONFIG_DWH_SCHEMA),
-]
+)
 # BigQuery config keys
-MF_BIGQUERY_KEYS = [
-    ConfigKey(key=CONFIG_DWH_PASSWORD, comment="Provide the path to the BigQuery credential file"),
-    ConfigKey(key=CONFIG_DWH_DIALECT, value="bigquery", comment="Dialect (one of BigQuery, Snowflake, Redshift)"),
-]
-# Redshift config keys
-MF_REDSHIFT_KEYS = [
+MF_BIGQUERY_KEYS = (
+    ConfigKey(
+        key=CONFIG_DWH_CREDS_PATH, comment="Provide the path to the BigQuery credential file, ignore to use ADC auth"
+    ),
+    ConfigKey(
+        key=CONFIG_DWH_PROJECT_ID, comment="Provide the GCP Project ID, ignore if using service account credentials"
+    ),
+    ConfigKey(key=CONFIG_DWH_DIALECT, value=SqlDialect.BIGQUERY.value),
+)
+
+# Postgres config keys
+MF_POSTGRESQL_KEYS = (
     ConfigKey(key=CONFIG_DWH_DB),
     ConfigKey(key=CONFIG_DWH_PASSWORD, comment="Password associated with the provided user"),
     ConfigKey(key=CONFIG_DWH_USER, comment="Username for the data warehouse"),
     ConfigKey(key=CONFIG_DWH_PORT),
     ConfigKey(key=CONFIG_DWH_HOST, comment="Host name"),
-    ConfigKey(key=CONFIG_DWH_DIALECT, value="redshift", comment="Dialect (one of BigQuery, Snowflake, Redshift)"),
-]
+    ConfigKey(key=CONFIG_DWH_DIALECT, value=SqlDialect.POSTGRESQL.value),
+)
+
 # Redshift config keys
-MF_SNOWFLAKE_KEYS = [
+MF_REDSHIFT_KEYS = (
+    ConfigKey(key=CONFIG_DWH_DB),
+    ConfigKey(key=CONFIG_DWH_PASSWORD, comment="Password associated with the provided user"),
+    ConfigKey(key=CONFIG_DWH_USER, comment="Username for the data warehouse"),
+    ConfigKey(key=CONFIG_DWH_PORT),
+    ConfigKey(key=CONFIG_DWH_HOST, comment="Host name"),
+    ConfigKey(key=CONFIG_DWH_DIALECT, value=SqlDialect.REDSHIFT.value),
+)
+# Snowflake config keys
+MF_SNOWFLAKE_KEYS = (
     ConfigKey(key=CONFIG_DWH_WAREHOUSE, comment="Provide the warehouse to use"),
     ConfigKey(key=CONFIG_DWH_DB),
     ConfigKey(key=CONFIG_DWH_PASSWORD, comment="Password associated with the provided user"),
     ConfigKey(key=CONFIG_DWH_USER, comment="Username for the data warehouse"),
     ConfigKey(key=CONFIG_DWH_HOST, comment="Snowflake account name"),
-    ConfigKey(key=CONFIG_DWH_DIALECT, value="snowflake", comment="Dialect (one of BigQuery, Snowflake, Redshift)"),
-]
+    ConfigKey(key=CONFIG_DWH_DIALECT, value=SqlDialect.SNOWFLAKE.value),
+)
+
+# Databricks config keys
+MF_DATABRICKS_KEYS = (
+    ConfigKey(key=CONFIG_DWH_HTTP_PATH),
+    ConfigKey(key=CONFIG_DWH_HOST),
+    ConfigKey(key=CONFIG_DWH_ACCESS_TOKEN),
+    ConfigKey(key=CONFIG_DWH_DIALECT, value=SqlDialect.DATABRICKS.value),
+)
+
+
+def generate_duckdb_demo_keys(config_dir: str) -> Tuple[ConfigKey, ...]:
+    """Generate configuration keys for DuckDB with a file in the config_dir."""
+    return (
+        ConfigKey(
+            key=CONFIG_DWH_DB, value=os.path.join(config_dir, "duck.db"), comment="For DuckDB, this is the data file."
+        ),
+        ConfigKey(key=CONFIG_DWH_DIALECT, value="duckdb"),
+        ConfigKey(key=CONFIG_DWH_SCHEMA, value="mf_demo"),
+    )
 
 
 # Click Options
 def query_options(function: Callable) -> Callable:
     """Common options for a query"""
-    function = separated_by_comma_option(
+    function = click.option(
         "--order",
-        help_msg='Metrics or dimensions to order by ("-" prefix for DESC). For example: --order -ds or --order ds,-revenue',
+        type=click_custom.SequenceParamType(),
+        help='Metrics or dimensions to order by ("-" prefix for DESC). For example: --order -ds or --order ds,-revenue',
         required=False,
     )(function)
     function = click.option(
@@ -82,13 +124,16 @@ def query_options(function: Callable) -> Callable:
         help='SQL-like where statement provided as a string. For example: --where "revenue > 100"',
     )(function)
     function = start_end_time_options(function)
-    function = separated_by_comma_option(
+    function = click.option(
         "--dimensions",
-        help_msg="Dimensions to group by: syntax is --dimensions ds or for multiple dimensions --dimensions ds,org",
+        type=click_custom.SequenceParamType(),
+        default="",
+        help="Dimensions to group by: syntax is --dimensions ds or for multiple dimensions --dimensions ds,org",
     )(function)
-    function = separated_by_comma_option(
+    function = click.option(
         "--metrics",
-        help_msg="Metrics to query for: syntax is --metrics bookings or for multiple metrics --metrics bookings,messages",
+        type=click_custom.SequenceParamType(),
+        help="Metrics to query for: syntax is --metrics bookings or for multiple metrics --metrics bookings,messages",
     )(function)
     return function
 
@@ -113,21 +158,6 @@ def start_end_time_options(function: Callable) -> Callable:
     return function
 
 
-def separated_by_comma_option(option_name: str, help_msg: str = "", required: bool = True) -> Callable:
-    """Parse input containing a string separated by commma to a List."""
-
-    def wraps(function: Callable) -> Callable:
-        function = click.option(
-            option_name,
-            required=required,
-            help=help_msg,
-            callback=lambda ctx, param, value: parse_comma_separated_inputs(value),
-        )(function)
-        return function
-
-    return wraps
-
-
 # Parsers/Validators
 def convert_to_datetime(datetime_str: Optional[str]) -> Optional[dt.datetime]:
     """Callback to convert string to datetime given as an iso8601 timestamp."""
@@ -187,15 +217,3 @@ def exception_handler(func: Callable[..., Any]) -> Callable[..., Any]:  # type:
             exit(1)
 
     return wrapper
-
-
-def build_validation_header_msg(level: ValidationIssueLevel) -> str:
-    """Builds the header message with colour."""
-    colour_map = {
-        ValidationIssueLevel.WARNING: "cyan",
-        ValidationIssueLevel.ERROR: "bright_red",
-        ValidationIssueLevel.FATAL: "bright_red",
-        ValidationIssueLevel.FUTURE_ERROR: "bright_yellow",
-    }
-
-    return click.style(level.name, bold=True, fg=colour_map[level])
