@@ -8,6 +8,10 @@ CONFIG_DWH_USER = "dwh_user"
 CONFIG_DWH_PASSWORD = "dwh_password"
 CONFIG_DWH_DB = "dwh_database"
 CONFIG_DWH_SCHEMA = "dwh_schema"
+CONFIG_DWH_CREDS_PATH = "dwh_path_to_creds"
 CONFIG_DWH_WAREHOUSE = "dwh_warehouse"
+CONFIG_DWH_PROJECT_ID = "dwh_project_id"
 CONFIG_EMAIL = "email"
 CONFIG_MODEL_PATH = "model_path"
+CONFIG_DWH_HTTP_PATH = "dwh_http_path"
+CONFIG_DWH_ACCESS_TOKEN = "dwh_access_token"
