@@ -4,11 +4,12 @@ import collections
 import logging
 import time
 from dataclasses import dataclass
-from typing import List, TypeVar, Optional, Generic, Dict, Tuple, Sequence, Set, Union
+from typing import DefaultDict, List, TypeVar, Optional, Generic, Dict, Tuple, Sequence, Set, Union
 
 from metricflow.constraints.time_constraint import TimeRangeConstraint
 from metricflow.dag.id_generation import IdGeneratorRegistry, DATAFLOW_PLAN_PREFIX
 from metricflow.dataflow.builder.costing import DefaultCostFunction, DataflowPlanNodeCostFunction
+from metricflow.dataflow.builder.measure_additiveness import group_measure_specs_by_additiveness
 from metricflow.dataflow.builder.node_data_set import DataflowPlanNodeOutputDataSetResolver
 from metricflow.dataflow.builder.node_evaluator import (
     NodeEvaluatorForLinkableInstances,
@@ -29,15 +30,17 @@ from metricflow.dataflow.dataflow_plan import (
     JoinOverTimeRangeNode,
     JoinToBaseOutputNode,
     OrderByLimitNode,
-    ReadSqlSourceNode,
     WhereConstraintNode,
     WriteToResultDataframeNode,
     WriteToResultTableNode,
+    SemiAdditiveJoinNode,
     SinkOutput,
 )
 from metricflow.dataflow.dataflow_plan_to_text import dataflow_dag_as_text
 from metricflow.dataflow.sql_table import SqlTable
+from metricflow.dataset.dataset import DataSet
 from metricflow.errors.errors import UnableToSatisfyQueryError
+from metricflow.model.spec_converters import WhereConstraintConverter
 from metricflow.model.objects.metric import MetricType, CumulativeMetricWindow
 from metricflow.model.semantic_model import SemanticModel
 from metricflow.object_utils import pformat_big_objects, assert_exactly_one_arg_set
@@ -45,9 +48,10 @@ from metricflow.plan_conversion.column_resolver import DefaultColumnAssociationR
 from metricflow.plan_conversion.node_processor import PreDimensionJoinNodeProcessor
 from metricflow.plan_conversion.sql_dataset import SqlDataSet
 from metricflow.plan_conversion.time_spine import TimeSpineSource
-from metricflow.query.query_parser import MetricFlowQueryParser
+from metricflow.references import TimeDimensionReference
 from metricflow.specs import (
     MetricSpec,
+    MetricInputMeasureSpec,
     LinkableInstanceSpec,
     MetricFlowQuerySpec,
     MeasureSpec,
@@ -55,10 +59,12 @@ from metricflow.specs import (
     IdentifierSpec,
     InstanceSpec,
     DimensionSpec,
-    TimeDimensionReference,
     OrderBySpec,
+    NonAdditiveDimensionSpec,
     SpecWhereClauseConstraint,
     LinkableSpecSet,
+    ColumnAssociationResolver,
+    LinklessIdentifierSpec,
 )
 from metricflow.time.time_granularity import TimeGranularity
 
@@ -82,27 +88,45 @@ class MeasureRecipe(Generic[SqlDataSetT]):
     join_linkable_instances_recipes: Tuple[JoinLinkableInstancesRecipe, ...]
 
 
+@dataclass(frozen=True)
+class MeasureSpecProperties:
+    """Input dataclass for grouping properties of a sequence of MeasureSpecs."""
+
+    measure_specs: Sequence[MeasureSpec]
+    data_source_name: str
+    agg_time_dimension: TimeDimensionReference
+    non_additive_dimension_spec: Optional[NonAdditiveDimensionSpec] = None
+
+
 class DataflowPlanBuilder(Generic[SqlDataSetT]):
     """Builds a dataflow plan to satisfy a given query."""
 
     def __init__(  # noqa: D
         self,
-        data_sets: List[SqlDataSetT],
+        source_nodes: Sequence[BaseOutput[SqlDataSetT]],
         semantic_model: SemanticModel,
         time_spine_source: TimeSpineSource,
-        primary_time_dimension_reference: TimeDimensionReference,
         cost_function: DataflowPlanNodeCostFunction = DefaultCostFunction[SqlDataSetT](),
+        node_output_resolver: Optional[DataflowPlanNodeOutputDataSetResolver[SqlDataSetT]] = None,
+        column_association_resolver: Optional[ColumnAssociationResolver] = None,
     ) -> None:
-        self._data_sets = data_sets
         self._data_source_semantics = semantic_model.data_source_semantics
         self._metric_semantics = semantic_model.metric_semantics
-        self._primary_time_dimension_reference = primary_time_dimension_reference
+        self._metric_time_dimension_reference = DataSet.metric_time_dimension_reference()
         self._cost_function = cost_function
-        self._source_nodes = [ReadSqlSourceNode[SqlDataSetT](x) for x in self._data_sets]
-        self._node_data_set_resolver = DataflowPlanNodeOutputDataSetResolver[SqlDataSetT](
-            column_association_resolver=DefaultColumnAssociationResolver(semantic_model),
-            semantic_model=semantic_model,
-            time_spine_source=time_spine_source,
+        self._source_nodes = source_nodes
+        self._node_data_set_resolver = (
+            DataflowPlanNodeOutputDataSetResolver[SqlDataSetT](
+                column_association_resolver=(
+                    DefaultColumnAssociationResolver(semantic_model)
+                    if not column_association_resolver
+                    else column_association_resolver
+                ),
+                semantic_model=semantic_model,
+                time_spine_source=time_spine_source,
+            )
+            if not node_output_resolver
+            else node_output_resolver
         )
 
     def build_plan(
@@ -129,21 +153,16 @@ class DataflowPlanBuilder(Generic[SqlDataSetT]):
         for metric_spec in query_spec.metric_specs:
             logger.info(f"Generating compute metrics node for {metric_spec}")
             metric = self._metric_semantics.get_metric(metric_spec)
-
-            measure_specs = tuple(
-                MeasureSpec(
-                    element_name=x.element_name,
-                )
-                for x in metric.measure_names
-            )
+            metric_input_measure_specs = self._metric_semantics.measures_for_metric(metric_spec)
 
             logger.info(
-                f"For {metric_spec}, needed measures are:\n" f"{pformat_big_objects(measure_specs=measure_specs)}"
+                f"For {metric_spec}, needed measures are:\n"
+                f"{pformat_big_objects(metric_input_measure_specs=metric_input_measure_specs)}"
             )
 
             combined_where = query_spec.where_constraint
             if metric.constraint:
-                metric_constraint = MetricFlowQueryParser.convert_to_spec_where_constraint(
+                metric_constraint = WhereConstraintConverter.convert_to_spec_where_constraint(
                     self._data_source_semantics, metric.constraint
                 )
                 if combined_where:
@@ -152,7 +171,7 @@ class DataflowPlanBuilder(Generic[SqlDataSetT]):
                     combined_where = metric_constraint
 
             aggregated_measures_node = self.build_aggregated_measures(
-                measure_specs=measure_specs,
+                metric_input_measure_specs=metric_input_measure_specs,
                 queried_linkable_specs=query_spec.linkable_specs,
                 where_constraint=combined_where,
                 time_range_constraint=query_spec.time_range_constraint,
@@ -214,7 +233,9 @@ class DataflowPlanBuilder(Generic[SqlDataSetT]):
             computed_metrics_output=distinct_values_node,
             order_by_specs=(
                 OrderBySpec(
-                    item=linkable_spec,
+                    dimension_spec=dimension_spec,
+                    time_dimension_spec=time_dimension_spec,
+                    identifier_spec=identifier_spec,
                     descending=False,
                 ),
             ),
@@ -288,7 +309,7 @@ class DataflowPlanBuilder(Generic[SqlDataSetT]):
 
         return sorted(nodes, key=sort_function)
 
-    def _select_source_nodes_with_meausres(
+    def _select_source_nodes_with_measures(
         self, measure_specs: Set[MeasureSpec], source_nodes: Sequence[BaseOutput[SqlDataSetT]]
     ) -> Sequence[BaseOutput[SqlDataSetT]]:
         nodes = []
@@ -301,9 +322,61 @@ class DataflowPlanBuilder(Generic[SqlDataSetT]):
                 nodes.append(source_node)
         return nodes
 
+    def _find_non_additive_dimension_in_linkable_specs(
+        self,
+        agg_time_dimension: TimeDimensionReference,
+        linkable_specs: Sequence[LinkableInstanceSpec],
+        non_additive_dimension_spec: NonAdditiveDimensionSpec,
+    ) -> Optional[TimeDimensionSpec]:
+        """Finds the TimeDimensionSpec matching the non_additive_dimension_spec, if any"""
+        queried_time_dimension_spec: Optional[LinkableInstanceSpec] = None
+        for linkable_spec in linkable_specs:
+            dimension_name_match = linkable_spec.element_name == non_additive_dimension_spec.name
+            metric_time_match = (
+                non_additive_dimension_spec.name == agg_time_dimension.element_name
+                and linkable_spec.element_name == self._metric_time_dimension_reference.element_name
+            )
+            if dimension_name_match or metric_time_match:
+                queried_time_dimension_spec = linkable_spec
+                break
+        assert queried_time_dimension_spec is None or isinstance(
+            queried_time_dimension_spec, TimeDimensionSpec
+        ), "Non-additive dimension can only be a time dimension, if specified."
+        return queried_time_dimension_spec
+
+    def _build_measure_spec_properties(self, measure_specs: Sequence[MeasureSpec]) -> MeasureSpecProperties:
+        """Ensures that the group of MeasureSpecs has the same non_additive_dimension_spec and agg_time_dimension"""
+        if len(measure_specs) == 0:
+            raise ValueError("Cannot build MeasureParametersForRecipe when given an empty sequence of measure_specs.")
+        data_sources = self._get_data_source_names_for_measures(measure_specs)
+        if len(data_sources) > 1:
+            raise ValueError(
+                f"Cannot find common properties for measures {measure_specs} coming from multiple "
+                f"data sources: {data_sources}. This suggests the measure_specs were not correctly filtered."
+            )
+
+        agg_time_dimension = agg_time_dimension = self._data_source_semantics.get_agg_time_dimension_for_measure(
+            measure_specs[0].as_reference
+        )
+        non_additive_dimension_spec = measure_specs[0].non_additive_dimension_spec
+        for measure_spec in measure_specs:
+            if non_additive_dimension_spec != measure_spec.non_additive_dimension_spec:
+                raise ValueError(f"measure_specs {measure_specs} do not have the same non_additive_dimension_spec.")
+            measure_agg_time_dimension = self._data_source_semantics.get_agg_time_dimension_for_measure(
+                measure_spec.as_reference
+            )
+            if measure_agg_time_dimension != agg_time_dimension:
+                raise ValueError(f"measure_specs {measure_specs} do not have the same agg_time_dimension.")
+        return MeasureSpecProperties(
+            measure_specs=measure_specs,
+            data_source_name=data_sources.pop(),
+            agg_time_dimension=agg_time_dimension,
+            non_additive_dimension_spec=non_additive_dimension_spec,
+        )
+
     def _find_measure_recipe(
         self,
-        measure_specs: Sequence[MeasureSpec],
+        measure_spec_properties: MeasureSpecProperties,
         linkable_specs: Sequence[LinkableInstanceSpec],
         time_range_constraint: Optional[TimeRangeConstraint] = None,
     ) -> Optional[MeasureRecipe]:
@@ -312,13 +385,7 @@ class DataflowPlanBuilder(Generic[SqlDataSetT]):
         Prior to calling this method we should always be checking that all input measure specs come from
         the same base data source, otherwise the internal conditions here will be impossible to satisfy
         """
-        data_sources = self._get_data_source_names_for_measures(measure_specs)
-        if len(data_sources) > 1:
-            raise ValueError(
-                f"Cannot find a single measure recipe for measures {measure_specs} coming from multiple "
-                f"data sources: {data_sources}. This suggests the measure_specs were not correctly filtered."
-            )
-
+        measure_specs = measure_spec_properties.measure_specs
         node_processor = PreDimensionJoinNodeProcessor(
             data_source_semantics=self._data_source_semantics,
             node_data_set_resolver=self._node_data_set_resolver,
@@ -327,7 +394,7 @@ class DataflowPlanBuilder(Generic[SqlDataSetT]):
         source_nodes: Sequence[BaseOutput[SqlDataSetT]] = self._source_nodes
 
         # We only care about nodes that have all required measures
-        potential_measure_nodes: Sequence[BaseOutput[SqlDataSetT]] = self._select_source_nodes_with_meausres(
+        potential_measure_nodes: Sequence[BaseOutput[SqlDataSetT]] = self._select_source_nodes_with_measures(
             measure_specs=set(measure_specs), source_nodes=source_nodes
         )
 
@@ -340,14 +407,14 @@ class DataflowPlanBuilder(Generic[SqlDataSetT]):
         if time_range_constraint:
             potential_measure_nodes = node_processor.add_time_range_constraint(
                 source_nodes=potential_measure_nodes,
-                primary_time_dimension_name=self._primary_time_dimension_reference.element_name,
+                metric_time_dimension_reference=self._metric_time_dimension_reference,
                 time_range_constraint=time_range_constraint,
             )
 
         nodes_available_for_joins = node_processor.remove_unnecessary_nodes(
             desired_linkable_specs=linkable_specs,
             nodes=source_nodes,
-            primary_time_dimension_reference=self._primary_time_dimension_reference,
+            metric_time_dimension_reference=self._metric_time_dimension_reference,
         )
         logger.info(
             f"After removing unnecessary nodes, there are {len(nodes_available_for_joins)} nodes available for joins"
@@ -355,7 +422,8 @@ class DataflowPlanBuilder(Generic[SqlDataSetT]):
         if DataflowPlanBuilder._contains_multihop_linkables(linkable_specs):
             nodes_available_for_joins = node_processor.add_multi_hop_joins(linkable_specs, source_nodes)
             logger.info(
-                f"After adding multi-hop nonds, there are {nodes_available_for_joins} nodes available for joins"
+                f"After adding multi-hop nodes, there are {len(nodes_available_for_joins)} nodes available for joins:\n"
+                f"{pformat_big_objects(nodes_available_for_joins)}"
             )
 
         logger.info(f"Processing nodes took: {time.time()-start_time:.2f}s")
@@ -387,7 +455,7 @@ class DataflowPlanBuilder(Generic[SqlDataSetT]):
                 start_node=node,
                 required_linkable_specs=list(linkable_specs),
             )
-            logger.info(f"Evaluation of {node} took {time.time() - start_time}s")
+            logger.info(f"Evaluation of {node} took {time.time() - start_time:.2f}s")
 
             logger.debug(
                 f"Evaluation for measure node is:\n"
@@ -469,7 +537,7 @@ class DataflowPlanBuilder(Generic[SqlDataSetT]):
 
     def build_aggregated_measures(
         self,
-        measure_specs: Tuple[MeasureSpec, ...],
+        metric_input_measure_specs: Tuple[MetricInputMeasureSpec, ...],
         queried_linkable_specs: LinkableSpecSet,
         where_constraint: Optional[SpecWhereClauseConstraint] = None,
         time_range_constraint: Optional[TimeRangeConstraint] = None,
@@ -484,75 +552,78 @@ class DataflowPlanBuilder(Generic[SqlDataSetT]):
         aggregated set of measures.
         """
         output_nodes: List[BaseOutput[SqlDataSetT]] = []
-        data_sources_for_measures = collections.defaultdict(list)
-        for measure_spec in measure_specs:
+        data_sources_and_constraints_to_measures: DefaultDict[
+            tuple[str, Optional[SpecWhereClauseConstraint]], List[MetricInputMeasureSpec]
+        ] = collections.defaultdict(list)
+        for input_spec in metric_input_measure_specs:
             data_source_names = [
                 dsource.name
                 for dsource in self._data_source_semantics.get_data_sources_for_measure(
-                    measure_reference=measure_spec.as_reference
+                    measure_reference=input_spec.measure_spec.as_reference
                 )
             ]
             assert (
                 len(data_source_names) == 1
-            ), f"Validation should enforce one data source per measure, but found {data_source_names} for {measure_spec}!"
-            data_sources_for_measures[data_source_names[0]].append(measure_spec)
+            ), f"Validation should enforce one data source per measure, but found {data_source_names} for {input_spec}!"
+            data_sources_and_constraints_to_measures[(data_source_names[0], input_spec.constraint)].append(input_spec)
 
-        for data_source, measures in data_sources_for_measures.items():
-            logger.info(f"Building aggregated measures for {data_source}. Measures: {measures}")
-            node = self._build_aggregated_measures_for_single_data_source(
-                measure_specs=tuple(measures),
-                queried_linkable_specs=queried_linkable_specs,
-                where_constraint=where_constraint,
-                time_range_constraint=time_range_constraint,
-                cumulative=cumulative,
-                cumulative_window=cumulative_window,
-                cumulative_grain_to_date=cumulative_grain_to_date,
+        for (data_source, measure_constraint), measures in data_sources_and_constraints_to_measures.items():
+            logger.info(
+                f"Building aggregated measures for {data_source}. "
+                f" Input measures: {measures} with constraints: {measure_constraint}"
             )
-            output_nodes.append(node)
+            if measure_constraint is None:
+                node_where_constraint = where_constraint
+            elif where_constraint is None:
+                node_where_constraint = measure_constraint
+            else:
+                node_where_constraint = where_constraint.combine(measure_constraint)
+
+            input_specs_by_measure_spec = {spec.measure_spec: spec for spec in measures}
+            grouped_measures_by_additiveness = group_measure_specs_by_additiveness(
+                tuple(input_specs_by_measure_spec.keys())
+            )
+            measures_by_additiveness = grouped_measures_by_additiveness.measures_by_additiveness
+
+            # Build output nodes for each distinct non-additive dimension spec, including the None case
+            for non_additive_spec, measure_specs in measures_by_additiveness.items():
+                non_additive_message = ""
+                if non_additive_spec is not None:
+                    non_additive_message = f" with non-additive dimension spec: {non_additive_spec}"
+
+                logger.info(f"Building aggregated measures for {data_source}{non_additive_message}")
+                input_specs = tuple(input_specs_by_measure_spec[measure_spec] for measure_spec in measure_specs)
+                output_nodes.append(
+                    self._build_aggregated_measures_from_measure_source_node(
+                        metric_input_measure_specs=input_specs,
+                        queried_linkable_specs=queried_linkable_specs,
+                        where_constraint=node_where_constraint,
+                        time_range_constraint=time_range_constraint,
+                        cumulative=cumulative,
+                        cumulative_window=cumulative_window,
+                        cumulative_grain_to_date=cumulative_grain_to_date,
+                    )
+                )
 
         if len(output_nodes) == 1:
             return output_nodes[0]
         else:
+            if len(queried_linkable_specs.as_tuple) == 0:
+                raise NotImplementedError(
+                    "Querying metrics that require a join with no dimensions is not yet supported."
+                    "This can happen with a metric that pulls measures from different data sources, "
+                    "or a metric that pulls measures with different constraints or non-additive dimension specifications."
+                )
             return FilterElementsNode(
                 parent_node=JoinAggregatedMeasuresByGroupByColumnsNode(parent_nodes=output_nodes),
-                include_specs=LinkableInstanceSpec.merge(queried_linkable_specs.as_tuple, measure_specs),
+                include_specs=LinkableInstanceSpec.merge(
+                    queried_linkable_specs.as_tuple, tuple(x.post_aggregation_spec for x in metric_input_measure_specs)
+                ),
             )
 
-    @staticmethod
-    def _add_where_and_aggregate(
-        measure_specs: Tuple[MeasureSpec, ...],
-        queried_linkable_specs: LinkableSpecSet,
-        unaggregated_node: BaseOutput[SqlDataSetT],
-        where_constraint: SpecWhereClauseConstraint,
-    ) -> BaseOutput[SqlDataSetT]:
-        """Applies the where constraint appropriately and produce aggregated measures.
-
-        Assumes that the input node has all measure specs, queried_linkable_specs, and also the specs required by the
-        where clause.
-        """
-        where_constrained_node = WhereConstraintNode(
-            parent_node=unaggregated_node,
-            where_constraint=where_constraint,
-        )
-
-        if where_constraint.linkable_spec_set.is_subset_of(queried_linkable_specs):
-            return AggregateMeasuresNode[SqlDataSetT](where_constrained_node)
-
-        # At this point, it's the case that the linkable specs in the where clause are not a subset of the queried
-        # linkable specs. A filter is needed after a where clause so that the linkable specs in the where clause don't
-        # show up in the final result.
-        #
-        # e.g. for "bookings" by "ds" where "is_instant", "is_instant" should not be in the results.
-        filtered_node = FilterElementsNode(
-            parent_node=where_constrained_node,
-            include_specs=measure_specs + queried_linkable_specs.as_tuple,
-        )
-
-        return AggregateMeasuresNode[SqlDataSetT](parent_node=filtered_node)
-
-    def _build_aggregated_measures_for_single_data_source(
+    def _build_aggregated_measures_from_measure_source_node(
         self,
-        measure_specs: Tuple[MeasureSpec, ...],
+        metric_input_measure_specs: Tuple[MetricInputMeasureSpec, ...],
         queried_linkable_specs: LinkableSpecSet,
         where_constraint: Optional[SpecWhereClauseConstraint] = None,
         time_range_constraint: Optional[TimeRangeConstraint] = None,
@@ -560,9 +631,13 @@ class DataflowPlanBuilder(Generic[SqlDataSetT]):
         cumulative_window: Optional[CumulativeMetricWindow] = None,
         cumulative_grain_to_date: Optional[TimeGranularity] = None,
     ) -> BaseOutput[SqlDataSetT]:
-        primary_time_dimension_requested = self._primary_time_dimension_reference.element_name in [
+        metric_time_dimension_requested = self._metric_time_dimension_reference.element_name in [
             linkable_spec.element_name for linkable_spec in queried_linkable_specs.as_tuple
         ]
+        measure_specs = tuple(x.measure_spec for x in metric_input_measure_specs)
+        measure_properties = self._build_measure_spec_properties(measure_specs)
+        non_additive_dimension_spec = measure_properties.non_additive_dimension_spec
+
         cumulative_metric_adjusted_time_constraint: Optional[TimeRangeConstraint] = None
         if cumulative and time_range_constraint is not None:
             logger.info(f"Time range constraint before adjustment is {time_range_constraint}")
@@ -580,12 +655,19 @@ class DataflowPlanBuilder(Generic[SqlDataSetT]):
             )
             logger.info(f"Adjusted time range constraint {cumulative_metric_adjusted_time_constraint}")
 
-        required_linkable_specs = (
-            LinkableSpecSet.merge((queried_linkable_specs, where_constraint.linkable_spec_set))
-            if where_constraint
-            else queried_linkable_specs
-        )
+        # Extraneous linkable specs are specs that are used in this phase that should not show up in the final result
+        # unless it was already a requested spec in the query
+        extraneous_linkable_specs = LinkableSpecSet()
+        if where_constraint:
+            extraneous_linkable_specs = LinkableSpecSet.merge(
+                (extraneous_linkable_specs, where_constraint.linkable_spec_set)
+            )
+        if non_additive_dimension_spec:
+            extraneous_linkable_specs = LinkableSpecSet.merge(
+                (extraneous_linkable_specs, non_additive_dimension_spec.linkable_specs)
+            )
 
+        required_linkable_specs = LinkableSpecSet.merge((queried_linkable_specs, extraneous_linkable_specs))
         logger.info(
             f"Looking for a recipe to get:\n"
             f"{pformat_big_objects(measure_specs=measure_specs, required_linkable_set=required_linkable_specs)}"
@@ -593,7 +675,7 @@ class DataflowPlanBuilder(Generic[SqlDataSetT]):
 
         find_recipe_start_time = time.time()
         measure_recipe = self._find_measure_recipe(
-            measure_specs=measure_specs,
+            measure_spec_properties=measure_properties,
             time_range_constraint=cumulative_metric_adjusted_time_constraint or time_range_constraint,
             linkable_specs=required_linkable_specs.as_tuple,
         )
@@ -620,7 +702,7 @@ class DataflowPlanBuilder(Generic[SqlDataSetT]):
         if cumulative:
             time_range_node = JoinOverTimeRangeNode(
                 parent_node=filtered_measure_source_node,
-                primary_time_dimension_reference=self._primary_time_dimension_reference,
+                metric_time_dimension_reference=self._metric_time_dimension_reference,
                 window=cumulative_window,
                 grain_to_date=cumulative_grain_to_date,
                 time_range_constraint=time_range_constraint,
@@ -637,7 +719,8 @@ class DataflowPlanBuilder(Generic[SqlDataSetT]):
 
             # If we're joining something in, then we need the associated identifier and partitions.
             include_specs: List[LinkableInstanceSpec] = [
-                x.identifier_links[0] for x in join_recipe.satisfiable_linkable_specs
+                LinklessIdentifierSpec.from_reference(x.identifier_links[0])
+                for x in join_recipe.satisfiable_linkable_specs
             ]
             include_specs.extend([x.node_to_join_dimension_spec for x in join_recipe.join_on_partition_dimensions])
             include_specs.extend(
@@ -684,20 +767,53 @@ class DataflowPlanBuilder(Generic[SqlDataSetT]):
         if (
             cumulative_metric_adjusted_time_constraint is not None
             and time_range_constraint is not None
-            and primary_time_dimension_requested
+            and metric_time_dimension_requested
         ):
             cumulative_metric_constrained_node = ConstrainTimeRangeNode(
                 unaggregated_measure_node, time_range_constraint
             )
 
+        pre_aggregate_node: BaseOutput[SqlDataSetT] = cumulative_metric_constrained_node or unaggregated_measure_node
         if where_constraint:
-            return DataflowPlanBuilder._add_where_and_aggregate(
-                measure_specs=measure_specs,
-                unaggregated_node=cumulative_metric_constrained_node or unaggregated_measure_node,
-                queried_linkable_specs=queried_linkable_specs,
+            # Apply where constraint on the node
+            pre_aggregate_node = WhereConstraintNode(
+                parent_node=pre_aggregate_node,
                 where_constraint=where_constraint,
             )
 
+        if non_additive_dimension_spec is not None:
+            # Apply semi additive join on the node
+            agg_time_dimension = measure_properties.agg_time_dimension
+            queried_time_dimension_spec: Optional[
+                TimeDimensionSpec
+            ] = self._find_non_additive_dimension_in_linkable_specs(
+                agg_time_dimension=agg_time_dimension,
+                linkable_specs=queried_linkable_specs.as_tuple,
+                non_additive_dimension_spec=non_additive_dimension_spec,
+            )
+            time_dimension_spec = TimeDimensionSpec.from_name(non_additive_dimension_spec.name)
+            window_groupings = tuple(
+                LinklessIdentifierSpec.from_element_name(name) for name in non_additive_dimension_spec.window_groupings
+            )
+            pre_aggregate_node = SemiAdditiveJoinNode[SqlDataSetT](
+                parent_node=pre_aggregate_node,
+                identifier_specs=window_groupings,
+                time_dimension_spec=time_dimension_spec,
+                agg_by_function=non_additive_dimension_spec.window_choice,
+                queried_time_dimension_spec=queried_time_dimension_spec,
+            )
+
+        if not extraneous_linkable_specs.is_subset_of(queried_linkable_specs):
+            # At this point, it's the case that the linkable specs in the extraneous specs are not a subset of the queried
+            # linkable specs. A filter is needed after, say, a where clause so that the linkable specs in the where clause don't
+            # show up in the final result.
+            #
+            # e.g. for "bookings" by "ds" where "is_instant", "is_instant" should not be in the results.
+            pre_aggregate_node = FilterElementsNode[SqlDataSetT](
+                parent_node=pre_aggregate_node,
+                include_specs=measure_specs + queried_linkable_specs.as_tuple,
+            )
         return AggregateMeasuresNode[SqlDataSetT](
-            parent_node=cumulative_metric_constrained_node or unaggregated_measure_node
+            parent_node=pre_aggregate_node,
+            metric_input_measure_specs=metric_input_measure_specs,
         )
