@@ -3,7 +3,7 @@ from __future__ import annotations
 import logging
 from collections import defaultdict
 from copy import deepcopy
-from typing import Dict, List, Set, Type, Optional, Sequence, Tuple
+from typing import Dict, List, Set, Optional, Sequence, Tuple, FrozenSet
 
 from metricflow.errors.errors import (
     DuplicateMetricError,
@@ -12,29 +12,33 @@ from metricflow.errors.errors import (
     InvalidDataSourceError,
 )
 from metricflow.instances import DataSourceReference, DataSourceElementReference
-from metricflow.model.graph import MultiDiGraph
-from metricflow.model.objects.common import Element
+from metricflow.aggregation_properties import AggregationType
 from metricflow.model.objects.data_source import DataSource, DataSourceOrigin
-from metricflow.model.objects.elements.dimension import Dimension, DimensionType
-from metricflow.model.objects.elements.measure import Measure, AggregationType
+from metricflow.model.objects.elements.dimension import Dimension
+from metricflow.model.objects.elements.identifier import Identifier
+from metricflow.model.objects.elements.measure import Measure
 from metricflow.model.objects.metric import Metric, MetricType
 from metricflow.model.objects.user_configured_model import UserConfiguredModel
 from metricflow.model.semantics.data_source_container import PydanticDataSourceContainer
+from metricflow.model.semantics.element_group import ElementGrouper
 from metricflow.model.semantics.linkable_spec_resolver import (
     ValidLinkableSpecResolver,
     LinkableElementProperties,
 )
-from metricflow.model.semantics.links_new import JoinLink, JOIN_TYPE_MAPPING
-from metricflow.specs import (
-    LinkableInstanceSpec,
-    LinkableElementReference,
-    MeasureSpec,
-    MeasureReference,
+from metricflow.model.spec_converters import MeasureConverter, WhereConstraintConverter
+from metricflow.references import (
     DimensionReference,
     IdentifierReference,
-    MetricSpec,
+    LinkableElementReference,
+    MeasureReference,
     TimeDimensionReference,
-    LinkableSpecSet,
+)
+from metricflow.specs import (
+    LinkableInstanceSpec,
+    MeasureSpec,
+    MetricInputMeasureSpec,
+    MetricSpec,
+    NonAdditiveDimensionSpec,
 )
 
 logger = logging.getLogger(__name__)
@@ -58,48 +62,23 @@ class MetricSemantics:  # noqa: D
 
         self._linkable_spec_resolver = ValidLinkableSpecResolver(
             user_configured_model=self._user_configured_model,
-            primary_time_dimension_reference=self._data_source_semantics.primary_time_dimension_reference,
             max_identifier_links=MAX_JOIN_HOPS,
         )
 
     def element_specs_for_metrics(
         self,
         metric_specs: List[MetricSpec],
-        local_only: bool = False,
-        dimensions_only: bool = False,
-        exclude_multi_hop: bool = False,
-        exclude_derived_time_granularities: bool = True,
-        exclude_local_linked_primary_time: bool = True,
+        with_any_property: FrozenSet[LinkableElementProperties] = LinkableElementProperties.all_properties(),
+        without_any_property: FrozenSet[LinkableElementProperties] = frozenset(),
     ) -> List[LinkableInstanceSpec]:
         """Dimensions common to all metrics requested (intersection)"""
-        without_any_property = set()
-
-        if exclude_local_linked_primary_time:
-            without_any_property.add(LinkableElementProperties.LOCAL_LINKED_PRIMARY_TIME)
-
-        if exclude_derived_time_granularities:
-            without_any_property.add(LinkableElementProperties.DERIVED_TIME_GRANULARITY)
-
-        if local_only:
-            with_any_property = frozenset({LinkableElementProperties.LOCAL})
-        else:
-            with_any_property = LinkableElementProperties.all_properties()
-
-        if exclude_multi_hop:
-            without_any_property.add(LinkableElementProperties.MULTI_HOP)
 
         all_linkable_specs = self._linkable_spec_resolver.get_linkable_elements_for_metrics(
             metric_specs=metric_specs,
             with_any_of=with_any_property,
-            without_any_of=frozenset(without_any_property),
+            without_any_of=without_any_property,
         ).as_spec_set
 
-        if dimensions_only:
-            all_linkable_specs = LinkableSpecSet(
-                dimension_specs=all_linkable_specs.dimension_specs,
-                time_dimension_specs=all_linkable_specs.time_dimension_specs,
-            )
-
         return sorted(all_linkable_specs.as_tuple, key=lambda x: x.qualified_name)
 
     def get_metrics(self, metric_names: List[MetricSpec]) -> List[Metric]:  # noqa: D
@@ -125,7 +104,7 @@ class MetricSemantics:  # noqa: D
         metric_spec = MetricSpec(element_name=metric.name)
         if metric_spec in self._metrics:
             raise DuplicateMetricError(f"Metric `{metric.name}` has already been registered")
-        for measure_reference in metric.measure_names:
+        for measure_reference in metric.measure_references:
             if measure_reference not in self._data_source_semantics.measure_references:
                 raise NonExistentMeasureError(
                     f"Metric `{metric.name}` references measure `{measure_reference}` which has not been registered"
@@ -138,16 +117,34 @@ class MetricSemantics:  # noqa: D
         """Return all of the hashes of the metric definitions."""
         return set(self._metric_hashes.values())
 
-    def measures_for_metric(self, metric_spec: MetricSpec) -> Tuple[MeasureSpec, ...]:
+    def measures_for_metric(self, metric_spec: MetricSpec) -> Tuple[MetricInputMeasureSpec, ...]:
         """Return the measure specs required to compute the metric."""
         metric = self.get_metric(metric_spec)
+        input_measure_specs: List[MetricInputMeasureSpec] = []
 
-        return tuple(
-            MeasureSpec(
-                element_name=x.element_name,
+        for input_measure in metric.input_measures:
+            spec_constraint = (
+                WhereConstraintConverter.convert_to_spec_where_constraint(
+                    data_source_semantics=self._data_source_semantics,
+                    where_constraint=input_measure.constraint,
+                )
+                if input_measure.constraint is not None
+                else None
             )
-            for x in metric.measure_names
-        )
+            measure_spec = MeasureSpec(
+                element_name=input_measure.name,
+                non_additive_dimension_spec=self._data_source_semantics.non_additive_dimension_specs_by_measure.get(
+                    input_measure.measure_reference
+                ),
+            )
+            spec = MetricInputMeasureSpec(
+                measure_spec=measure_spec,
+                constraint=spec_constraint,
+                alias=input_measure.alias,
+            )
+            input_measure_specs.append(spec)
+
+        return tuple(input_measure_specs)
 
     def contains_cumulative_metric(self, metric_specs: Sequence[MetricSpec]) -> bool:
         """Returns true if any of the specs correspond to a cumulative metric."""
@@ -158,7 +155,11 @@ class MetricSemantics:  # noqa: D
 
 
 class DataSourceSemantics:
-    """Tracks semantic information for data source held in a set of DataSourceContainers"""
+    """Tracks semantic information for data source held in a set of DataSourceContainers
+
+    This implements both the DataSourceSemanticsAccessors protocol, the interface type we use throughout the codebase.
+    That interface prevents unwanted calls to methods for adding data sources to the container.
+    """
 
     def __init__(  # noqa: D
         self,
@@ -166,66 +167,36 @@ class DataSourceSemantics:
         configured_data_source_container: PydanticDataSourceContainer,
     ) -> None:
         self._model = model
-        self._element_types: Dict[LinkableElementReference, Type[Element]] = {}
         self._measure_index: Dict[MeasureReference, List[DataSource]] = defaultdict(list)
         self._measure_aggs: Dict[
             MeasureReference, AggregationType
         ] = {}  # maps measures to their one consistent aggregation
+        self._measure_agg_time_dimension: Dict[MeasureReference, TimeDimensionReference] = {}
+        self._measure_non_additive_dimension_specs: Dict[MeasureReference, NonAdditiveDimensionSpec] = {}
         self._dimension_index: Dict[DimensionReference, List[DataSource]] = defaultdict(list)
         self._linkable_reference_index: Dict[LinkableElementReference, List[DataSource]] = defaultdict(list)
         self._entity_index: Dict[Optional[str], List[DataSource]] = defaultdict(list)
         self._identifier_ref_to_entity: Dict[IdentifierReference, Optional[str]] = {}
-        # Graph where nodes are DataSource names and edges are Join objects that represent all valid
-        # joins possible between the nodes (data sources)
-        self._data_source_links = MultiDiGraph[str, JoinLink]()
-
         self._data_source_names: Set[str] = set()
 
         self._configured_data_source_container = configured_data_source_container
+        self._data_source_to_aggregation_time_dimensions: Dict[
+            DataSourceReference, ElementGrouper[TimeDimensionReference, MeasureSpec]
+        ] = {}
 
         # Add semantic tracking for data sources from configured_data_source_container
         for data_source in self._configured_data_source_container.values():
             assert isinstance(data_source, DataSource)
             self.add_configured_data_source(data_source)
 
-    @property
-    def data_sources(self) -> Sequence[DataSource]:  # noqa: D
-        return self._model.data_sources
-
     def add_configured_data_source(self, data_source: DataSource) -> None:
         """Dont use this unless you mean it (ie in tests). The configured data sources are supposed to be static"""
         self._configured_data_source_container._put(data_source)
-        self.add_data_source(data_source)
-
-    def get_linkable_element_references(self) -> List[LinkableElementReference]:  # noqa: D
-        return list(self._linkable_reference_index.keys())
+        self._add_data_source(data_source)
 
     def get_dimension_references(self) -> List[DimensionReference]:  # noqa: D
         return list(self._dimension_index.keys())
 
-    def get_linkable(  # noqa: D
-        self,
-        linkable_name: LinkableElementReference,
-        origin: Optional[DataSourceOrigin] = None,
-        use_identifier_dundered_name: bool = False,
-    ) -> Element:
-        """Retrieves a full linkable element object by name
-
-        use_identifier_dundered_name returns the Dimension with the identifier-dundered name (if the requested linkable element
-        has an identifier-dunder) otherwise the identifier is stripped from the name
-        """
-        for dimension_source in self._linkable_reference_index[linkable_name]:
-            if origin and dimension_source.origin != origin:
-                continue
-            linkable_element = dimension_source.get_element(linkable_name)
-            # find the data source that has the requested linkable_element by the requested identifier
-
-            return deepcopy(linkable_element)
-
-        raise ValueError(
-            f"Could not find dimension with name ({linkable_name.element_name}) in configured data sources"
-        )
-
     def get_dimension(
         self, dimension_reference: DimensionReference, origin: Optional[DataSourceOrigin] = None
     ) -> Dimension:
@@ -242,10 +213,30 @@ class DataSourceSemantics:
             f"Could not find dimension with name ({dimension_reference.element_name}) in configured data sources"
         )
 
+    def get_time_dimension(self, time_dimension_reference: TimeDimensionReference) -> Dimension:
+        """Retrieves a full dimension object by name"""
+        dimension_reference = time_dimension_reference.dimension_reference()
+
+        if dimension_reference not in self._dimension_index:
+            raise ValueError(
+                f"Could not find dimension with name ({dimension_reference.element_name}) in configured data sources"
+            )
+
+        for dimension_source in self._dimension_index[dimension_reference]:
+            dimension = dimension_source.get_dimension(dimension_reference)
+            # TODO: Unclear if the deepcopy is necessary.
+            return deepcopy(dimension)
+
+        assert False, f"{time_dimension_reference} should have been in the dimension index"
+
     @property
     def measure_references(self) -> List[MeasureReference]:  # noqa: D
         return list(self._measure_index.keys())
 
+    @property
+    def non_additive_dimension_specs_by_measure(self) -> Dict[MeasureReference, NonAdditiveDimensionSpec]:  # noqa: D
+        return self._measure_non_additive_dimension_specs
+
     def get_measure(self, measure_reference: MeasureReference) -> Measure:  # noqa: D
         if measure_reference not in self._measure_index:
             raise ValueError(f"Could not find measure with name ({measure_reference}) in configured data sources")
@@ -257,47 +248,26 @@ class DataSourceSemantics:
     def get_identifier_references(self) -> List[IdentifierReference]:  # noqa: D
         return list(self._identifier_ref_to_entity.keys())
 
-    def _get_joinable_data_sources_by_identifier(
-        self, data_source_name: str, identifier_spec: LinkableInstanceSpec
-    ) -> List[str]:
-        """Returns list of data sources that can be joined to `data_source_name` via `identifier_spec`"""
-        res = []
-        for name, links in self._data_source_links.adj(data_source_name).items():
-            for link in links:
-                if link.via_from.name.element_name == identifier_spec.element_name:
-                    res.append(name)
-                    break
-
-        return res
-
-    def _get_joinable_data_sources(self, data_source_name: str) -> Dict[str, List[JoinLink]]:  # noqa: D
-        return self._data_source_links.adj(data_source_name)
-
     # DSC interface
     def get_data_sources_for_measure(self, measure_reference: MeasureReference) -> List[DataSource]:  # noqa: D
         return self._measure_index[measure_reference]
 
-    def dimension_is_partitioned(self, dimension_reference: DimensionReference) -> bool:  # noqa: D
-        return self.get_dimension(dimension_reference).is_partition
+    def get_agg_time_dimension_for_measure(  # noqa: D
+        self, measure_reference: MeasureReference
+    ) -> TimeDimensionReference:
+        return self._measure_agg_time_dimension[measure_reference]
 
-    def keys(self) -> List[str]:  # noqa: D
-        return list(self._data_source_names)
+    def get_identifier_in_data_source(self, ref: DataSourceElementReference) -> Optional[Identifier]:  # Noqa: d
+        data_source = self.get(ref.data_source_name)
+        if not data_source:
+            return None
 
-    def get_data_source_element(self, ref: DataSourceElementReference) -> Optional[Element]:  # Noqa: d
-        data_source = self[ref.data_source_name]
-        for elem in data_source.elements:
-            if elem.name.element_name == ref.element_name:
-                return elem
+        for identifier in data_source.identifiers:
+            if identifier.reference.element_name == ref.element_name:
+                return identifier
 
         return None
 
-    def __getitem__(self, item: str) -> DataSource:  # noqa: D
-        res = self.get(item)
-        if res:
-            return res
-
-        raise ValueError(f"Cannot find data source with name ({item})")
-
     def get(self, data_source_name: str) -> Optional[DataSource]:  # noqa: D
         if data_source_name in self._configured_data_source_container:
             data_source = self._configured_data_source_container.get(data_source_name)
@@ -309,28 +279,23 @@ class DataSourceSemantics:
     def get_by_reference(self, data_source_reference: DataSourceReference) -> Optional[DataSource]:  # noqa: D
         return self.get(data_source_reference.data_source_name)
 
-    def add_data_source(
+    def _add_data_source(
         self,
         data_source: DataSource,
         fail_on_error: bool = True,
         logging_context: str = "",
     ) -> None:
-        """Add data source semantic information, validating consistency with existing data sources and establishing links"""
+        """Add data source semantic information, validating consistency with existing data sources."""
         errors = []
 
         if data_source.name in self._data_source_names:
             errors.append(f"name {data_source.name} already registered - please ensure data source names are unique")
 
-        if data_source.origin is DataSourceOrigin.DERIVED:
-            raise ValueError(
-                f"Cannot add derived data source (with name: {data_source.name}) to New DataSourceSemantics... for now"
-            )
-
         for measure in data_source.measures:
-            if measure.name in self._measure_aggs and self._measure_aggs[measure.name] != measure.agg:
+            if measure.reference in self._measure_aggs and self._measure_aggs[measure.reference] != measure.agg:
                 errors.append(
-                    f"conflicting aggregation (agg) for measure `{measure.name}` registered as `{self._measure_aggs[measure.name]}`; "
-                    f"Got `{measure.agg}"
+                    f"conflicting aggregation (agg) for measure `{measure.reference.element_name}` registered as "
+                    f"`{self._measure_aggs[measure.reference]}`; Got `{measure.agg}"
                 )
 
         if errors:
@@ -347,81 +312,49 @@ class DataSourceSemantics:
             return
 
         self._data_source_names.add(data_source.name)
+        self._data_source_to_aggregation_time_dimensions[data_source.reference] = ElementGrouper[
+            TimeDimensionReference, MeasureSpec
+        ]()
 
-        for elem in data_source.elements:
-            if elem.name not in self._element_types:
-                self._element_types[elem.name] = type(elem)
-
-        for meas in data_source.measures:
-            self._measure_aggs[meas.name] = meas.agg
-            self._measure_index[meas.name].append(data_source)
+        for measure in data_source.measures:
+            self._measure_aggs[measure.reference] = measure.agg
+            self._measure_index[measure.reference].append(data_source)
+            agg_time_dimension = measure.checked_agg_time_dimension
+            self._data_source_to_aggregation_time_dimensions[data_source.reference].add_value(
+                key=agg_time_dimension,
+                value=MeasureConverter.convert_to_measure_spec(measure=measure),
+            )
+            self._measure_agg_time_dimension[measure.reference] = agg_time_dimension
+            if measure.non_additive_dimension:
+                non_additive_dimension_spec = NonAdditiveDimensionSpec(
+                    name=measure.non_additive_dimension.name,
+                    window_choice=measure.non_additive_dimension.window_choice,
+                    window_groupings=tuple(measure.non_additive_dimension.window_groupings),
+                )
+                self._measure_non_additive_dimension_specs[measure.reference] = non_additive_dimension_spec
         for dim in data_source.dimensions:
-            self._linkable_reference_index[dim.name].append(data_source)
-            self._dimension_index[dim.name].append(data_source)
+            self._linkable_reference_index[dim.reference].append(data_source)
+            self._dimension_index[dim.reference].append(data_source)
         for ident in data_source.identifiers:
-            self._identifier_ref_to_entity[ident.name] = ident.entity
+            self._identifier_ref_to_entity[ident.reference] = ident.entity
             self._entity_index[ident.entity].append(data_source)
-            self._linkable_reference_index[ident.name].append(data_source)
-
-        dsource_partition = data_source.partition
-        # Add links to other data sources based on the names and types of identifiers available.
-        for ident in data_source.identifiers:
-            # We only need to consider other data sources with the same identifier.
-            for other in self._entity_index[ident.entity]:
-                if other.name == data_source.name:
-                    continue
-
-                # handle whether or not this join is partitioned
-                partitions: List[DimensionReference] = list()
-                other_partition = other.partition
-                if dsource_partition and other_partition and dsource_partition.name != other_partition.name:
-                    continue
-                if dsource_partition and other_partition:
-                    partitions.append(dsource_partition.name)
-
-                for other_ident in other.identifiers:
-                    # TODO: Replace with entity check when entities are supported.
-                    if other_ident.name != ident.name:
-                        continue
-
-                    # add edge data_source -> other "other is joinable to data_source"
-                    join_type = JOIN_TYPE_MAPPING.get((ident.type, other_ident.type), None)
-                    if join_type:
-                        self._data_source_links.add_edge(
-                            from_key=data_source.name,
-                            to_key=other.name,
-                            value=JoinLink(
-                                join_type=join_type,
-                                via_from=ident,
-                                via_to=other_ident,
-                                partitions=tuple([str(p) for p in partitions]),
-                            ),
-                        )
-
-                    # add edge other -> data_source "data_source is joinable to other"
-                    reversed_join_type = JOIN_TYPE_MAPPING.get((other_ident.type, ident.type), None)
-                    if reversed_join_type:
-                        self._data_source_links.add_edge(
-                            from_key=other.name,
-                            to_key=data_source.name,
-                            value=JoinLink(
-                                join_type=reversed_join_type,
-                                via_from=other_ident,
-                                via_to=ident,
-                                partitions=tuple([str(p) for p in partitions]),
-                            ),
-                        )
-
-    @property
-    def primary_time_dimension_reference(self) -> TimeDimensionReference:
-        """Gets the primary time dimension that's used in all data sources."""
-        for data_source in self._configured_data_source_container.values():
-            for dimension in data_source.dimensions:
-                if dimension.type == DimensionType.TIME and dimension.type_params and dimension.type_params.is_primary:
-                    return TimeDimensionReference(element_name=dimension.name.element_name)
-        raise RuntimeError("No primary time dimension found in the model")
+            self._linkable_reference_index[ident.reference].append(data_source)
 
     @property
     def data_source_references(self) -> Sequence[DataSourceReference]:  # noqa: D
         data_source_names_sorted = sorted(self._data_source_names)
         return tuple(DataSourceReference(data_source_name=x) for x in data_source_names_sorted)
+
+    def get_aggregation_time_dimensions_with_measures(
+        self, data_source_reference: DataSourceReference
+    ) -> ElementGrouper[TimeDimensionReference, MeasureSpec]:
+        """Return all time dimensions in a data source with their associated measures."""
+        assert (
+            data_source_reference in self._data_source_to_aggregation_time_dimensions
+        ), f"Data Source {data_source_reference} is not known"
+        return self._data_source_to_aggregation_time_dimensions[data_source_reference]
+
+    def get_data_sources_for_identifier(self, identifier_reference: IdentifierReference) -> Set[DataSource]:
+        """Return all data sources associated with an identifier reference"""
+        identifier_entity = self._identifier_ref_to_entity[identifier_reference]
+        return set(self._entity_index[identifier_entity])
