@@ -10,13 +10,19 @@ import fuzzywuzzy.fuzz
 import fuzzywuzzy.process
 
 from metricflow.constraints.time_constraint import TimeRangeConstraint
+from metricflow.dataflow.builder.node_data_set import DataflowPlanNodeOutputDataSetResolver
+from metricflow.dataflow.dataflow_plan import BaseOutput
+from metricflow.dataset.data_source_adapter import DataSourceDataSet
+from metricflow.dataset.dataset import DataSet
+from metricflow.errors.errors import UnableToSatisfyQueryError
 from metricflow.model.objects.constraints.where import WhereClauseConstraint
 from metricflow.model.objects.elements.dimension import DimensionType
-from metricflow.model.objects.elements.identifier import IdentifierType
 from metricflow.model.semantic_model import SemanticModel
+from metricflow.model.spec_converters import WhereConstraintConverter
 from metricflow.naming.linkable_spec_name import StructuredLinkableSpecName
 from metricflow.object_utils import pformat_big_objects
 from metricflow.query.query_exceptions import InvalidQueryException
+from metricflow.references import DimensionReference, IdentifierReference, TimeDimensionReference
 from metricflow.specs import (
     MetricFlowQuerySpec,
     MetricSpec,
@@ -24,23 +30,17 @@ from metricflow.specs import (
     TimeDimensionSpec,
     IdentifierSpec,
     LinkableInstanceSpec,
-    LinklessIdentifierSpec,
     OrderBySpec,
-    TimeDimensionReference,
-    DimensionReference,
-    IdentifierReference,
     OutputColumnNameOverride,
     SpecWhereClauseConstraint,
     LinkableSpecSet,
 )
+from metricflow.time.time_granularity import TimeGranularity
 from metricflow.time.time_granularity_solver import (
     TimeGranularitySolver,
     PartialTimeDimensionSpec,
     RequestTimeGranularityException,
 )
-from metricflow.time.time_granularity import TimeGranularity
-from metricflow.errors.errors import UnableToSatisfyQueryError
-from metricflow.model.semantics.semantic_containers import DataSourceSemantics
 
 logger = logging.getLogger(__name__)
 
@@ -65,16 +65,14 @@ class MetricFlowQueryParser:
     "user_id__country" is the "country" dimension that is retrieved by joining "user_id" to the measure data source.
     """
 
-    # TODO: Separate methods out of MetricFlowQueryParser
-    # https://app.asana.com/0/1161293048858925/1201755441255186
-
     # Prefix to use for indents when logging.
     _INDENT_PREFIX = "  "
 
     def __init__(  # noqa: D
         self,
         model: SemanticModel,
-        primary_time_dimension_reference: TimeDimensionReference,
+        source_nodes: Sequence[BaseOutput[DataSourceDataSet]],
+        node_output_resolver: DataflowPlanNodeOutputDataSetResolver[DataSourceDataSet],
     ) -> None:
         self._model = model
         self._metric_semantics = model.metric_semantics
@@ -83,85 +81,23 @@ class MetricFlowQueryParser:
         # Set up containers for known element names
         self._known_identifier_element_references = self._data_source_semantics.get_identifier_references()
 
-        self._known_time_dimension_element_references = []
+        self._known_time_dimension_element_references = [DataSet.metric_time_dimension_reference()]
         self._known_dimension_element_references = []
-        for linkable_name in self._data_source_semantics.get_linkable_element_references():
-            linkable = self._data_source_semantics.get_linkable(linkable_name)
-            if linkable.type == DimensionType.CATEGORICAL:
-                self._known_dimension_element_references.append(linkable.name)
-            elif linkable.type == DimensionType.TIME:
-                self._known_time_dimension_element_references.append(linkable.name)
-            elif (
-                linkable.type == IdentifierType.FOREIGN
-                or linkable.type == IdentifierType.PRIMARY
-                or linkable.type == IdentifierType.UNIQUE
-            ):
-                pass
+        for dimension_reference in self._data_source_semantics.get_dimension_references():
+            dimension = self._data_source_semantics.get_dimension(dimension_reference)
+            if dimension.type == DimensionType.CATEGORICAL:
+                self._known_dimension_element_references.append(dimension_reference)
+            elif dimension.type == DimensionType.TIME:
+                self._known_time_dimension_element_references.append(dimension_reference.time_dimension_reference)
             else:
-                raise RuntimeError(f"Unhandled linkable type: {linkable.type}")
+                raise RuntimeError(f"Unhandled linkable type: {dimension.type}")
 
         self._known_metric_names = set(self._metric_semantics.metric_names)
-        self._primary_time_dimension_reference = primary_time_dimension_reference
-        self._time_granularity_solver = TimeGranularitySolver(self._model)
-
-    @staticmethod
-    def convert_to_linkable_specs(
-        data_source_semantics: DataSourceSemantics, where_constraint_names: List[str]
-    ) -> LinkableSpecSet:
-        """Processes where_clause_constraint.linkable_names into associated LinkableInstanceSpecs (dims, times, ids)
-
-        where_constraint_names: WhereConstraintClause.linkable_names
-        data_source_semantics: DataSourceSemantics from the instantiated class
-
-        output: InstanceSpecSet of Tuple(DimensionSpec), Tuple(TimeDimensionSpec), Tuple(IdentifierSpec)
-        """
-        where_constraint_dimensions = []
-        where_constraint_time_dimensions = []
-        where_constraint_identifiers = []
-        linkable_spec_names = [
-            StructuredLinkableSpecName.from_name(linkable_name) for linkable_name in where_constraint_names
-        ]
-        dimension_references = {
-            dimension_reference.element_name: dimension_reference
-            for dimension_reference in data_source_semantics.get_dimension_references()
-        }
-        identifier_references = {
-            identifier_reference.element_name: identifier_reference
-            for identifier_reference in data_source_semantics.get_identifier_references()
-        }
-
-        for spec_name in linkable_spec_names:
-            if spec_name.element_name in dimension_references:
-                dimension = data_source_semantics.get_dimension(dimension_references[spec_name.element_name])
-                if dimension.type == DimensionType.CATEGORICAL:
-                    where_constraint_dimensions.append(DimensionSpec.parse(spec_name.qualified_name))
-                elif dimension.type == DimensionType.TIME:
-                    where_constraint_time_dimensions.append(TimeDimensionSpec.parse(spec_name.qualified_name))
-                else:
-                    raise RuntimeError(f"Unhandled type: {dimension.type}")
-            elif spec_name.element_name in identifier_references:
-                where_constraint_identifiers.append(IdentifierSpec.parse(spec_name.qualified_name))
-            else:
-                raise InvalidQueryException(f"Unknown element: {spec_name}")
-
-        return LinkableSpecSet(
-            dimension_specs=tuple(where_constraint_dimensions),
-            time_dimension_specs=tuple(where_constraint_time_dimensions),
-            identifier_specs=tuple(where_constraint_identifiers),
-        )
-
-    @staticmethod
-    def convert_to_spec_where_constraint(
-        data_source_semantics: DataSourceSemantics, where_constraint: WhereClauseConstraint
-    ) -> SpecWhereClauseConstraint:
-        """Converts a where constraint to one using specs."""
-        return SpecWhereClauseConstraint(
-            where_condition=where_constraint.where,
-            linkable_names=tuple(where_constraint.linkable_names),
-            linkable_spec_set=MetricFlowQueryParser.convert_to_linkable_specs(
-                data_source_semantics=data_source_semantics, where_constraint_names=where_constraint.linkable_names
-            ),
-            execution_parameters=where_constraint.sql_params,
+        self._metric_time_dimension_reference = DataSet.metric_time_dimension_reference()
+        self._time_granularity_solver = TimeGranularitySolver(
+            semantic_model=self._model,
+            source_nodes=source_nodes,
+            node_output_resolver=node_output_resolver,
         )
 
     @staticmethod
@@ -237,13 +173,7 @@ class MetricFlowQueryParser:
         if len(invalid_group_bys) > 0:
             valid_group_by_names_for_metrics = sorted(
                 x.qualified_name
-                for x in self._metric_semantics.element_specs_for_metrics(
-                    metric_specs=list(metric_specs),
-                    local_only=False,
-                    dimensions_only=False,
-                    exclude_multi_hop=False,
-                    exclude_derived_time_granularities=False,
-                )
+                for x in self._metric_semantics.element_specs_for_metrics(metric_specs=list(metric_specs))
             )
             # Create suggestions for invalid dimensions in case the user made a typo.
             suggestion_sections = {}
@@ -278,6 +208,9 @@ class MetricFlowQueryParser:
             where_constraint and where_constraint_str
         ), "Both where_constraint and where_constraint_str should not be set"
 
+        if len(group_by_names) == 0:
+            assert len(metric_names) == 1, "Querying for multiple metrics with no dimensions is not yet supported."
+
         parsed_where_constraint: Optional[WhereClauseConstraint]
         if where_constraint_str:
             parsed_where_constraint = WhereClauseConstraint.parse(where_constraint_str)
@@ -322,14 +255,19 @@ class MetricFlowQueryParser:
                 f"{time_constraint_start.isoformat()}"
             )
 
-        time_constraint = TimeRangeConstraint(start_time=time_constraint_start, end_time=time_constraint_end)
+        time_constraint: Optional[TimeRangeConstraint] = TimeRangeConstraint(
+            start_time=time_constraint_start, end_time=time_constraint_end
+        )
+        if time_constraint == TimeRangeConstraint.all_time():
+            # If the time constraint is all time, just ignore and not render
+            time_constraint = None
 
         requested_linkable_specs = self._parse_linkable_element_names(group_by_names, metric_specs)
         partial_time_dimension_spec_replacements = (
             self._time_granularity_solver.resolve_granularity_for_partial_time_dimension_specs(
                 metric_specs=metric_specs,
                 partial_time_dimension_specs=requested_linkable_specs.partial_time_dimension_specs,
-                primary_time_dimension_reference=self._primary_time_dimension_reference,
+                metric_time_dimension_reference=self._metric_time_dimension_reference,
                 time_granularity=time_granularity,
             )
         )
@@ -392,19 +330,19 @@ class MetricFlowQueryParser:
         # time dimension. We should aim to get rid of this logic.
         output_column_name_overrides = []
         if (
-            self._primary_time_dimension_specified_without_granularity(
+            self._metric_time_dimension_specified_without_granularity(
                 requested_linkable_specs.partial_time_dimension_specs
             )
             and not time_granularity
         ):
             if self._metrics_have_same_time_granularities(metric_specs):
-                _, replace_with_time_dimension_spec = self._find_replacement_for_primary_time_dimension(
+                _, replace_with_time_dimension_spec = self._find_replacement_for_metric_time_dimension(
                     partial_time_dimension_spec_replacements
                 )
                 output_column_name_overrides.append(
                     OutputColumnNameOverride(
                         time_dimension_spec=replace_with_time_dimension_spec,
-                        output_column_name=self._primary_time_dimension_reference.element_name,
+                        output_column_name=self._metric_time_dimension_reference.element_name,
                     )
                 )
 
@@ -413,14 +351,9 @@ class MetricFlowQueryParser:
 
         spec_where_constraint: Optional[SpecWhereClauseConstraint] = None
         if parsed_where_constraint:
-            spec_where_constraint = SpecWhereClauseConstraint(
-                where_condition=parsed_where_constraint.where,
-                linkable_names=tuple(parsed_where_constraint.linkable_names),
-                linkable_spec_set=MetricFlowQueryParser.convert_to_linkable_specs(
-                    data_source_semantics=self._data_source_semantics,
-                    where_constraint_names=parsed_where_constraint.linkable_names,
-                ),
-                execution_parameters=parsed_where_constraint.sql_params,
+            spec_where_constraint = WhereConstraintConverter.convert_to_spec_where_constraint(
+                data_source_semantics=self._data_source_semantics,
+                where_constraint=parsed_where_constraint,
             )
 
         return MetricFlowQuerySpec(
@@ -461,7 +394,7 @@ class MetricFlowQueryParser:
         """Adjust the time range constraint so that it matches the boundaries of the granularity of the result."""
         self._time_granularity_solver.validate_time_granularity(metric_specs, time_dimension_specs)
 
-        smallest_primary_time_granularity_in_query = self._find_smallest_primary_time_dimension_spec_granularity(
+        smallest_primary_time_granularity_in_query = self._find_smallest_metric_time_dimension_spec_granularity(
             time_dimension_specs
         )
         if smallest_primary_time_granularity_in_query:
@@ -470,36 +403,34 @@ class MetricFlowQueryParser:
         else:
             _, adjusted_to_granularity = self._time_granularity_solver.local_dimension_granularity_range(
                 metric_specs=metric_specs,
-                local_time_dimension_reference=self._primary_time_dimension_reference,
+                local_time_dimension_reference=self._metric_time_dimension_reference,
             )
         logger.info(f"Adjusted primary time granularity is {adjusted_to_granularity}")
         return self._time_granularity_solver.adjust_time_range_to_granularity(
             time_range_constraint, adjusted_to_granularity
         )
 
-    def _find_replacement_for_primary_time_dimension(
+    def _find_replacement_for_metric_time_dimension(
         self, replacements: Dict[PartialTimeDimensionSpec, TimeDimensionSpec]
     ) -> Tuple[PartialTimeDimensionSpec, TimeDimensionSpec]:
         for partial_time_dimension_spec_to_replace, replace_with_time_dimension_spec in replacements.items():
             if (
                 partial_time_dimension_spec_to_replace.element_name
-                == self._primary_time_dimension_reference.element_name
+                == self._metric_time_dimension_reference.element_name
                 and partial_time_dimension_spec_to_replace.identifier_links == ()
             ):
                 return partial_time_dimension_spec_to_replace, replace_with_time_dimension_spec
 
-        raise RuntimeError(
-            f"Replacement for primary time dimension '{self._primary_time_dimension_reference}' not found"
-        )
+        raise RuntimeError(f"Replacement for metric time dimension '{self._metric_time_dimension_reference}' not found")
 
-    def _primary_time_dimension_specified_without_granularity(
+    def _metric_time_dimension_specified_without_granularity(
         self, partial_time_dimension_specs: Sequence[PartialTimeDimensionSpec]
     ) -> bool:
         # This detects a user query like: "query --metrics monthly_bookings --dimensions ds"
         # The granularity for "ds" is not specified by the user.
         for time_dimension_spec in partial_time_dimension_specs:
             if (
-                time_dimension_spec.element_name == self._primary_time_dimension_reference.element_name
+                time_dimension_spec.element_name == self._metric_time_dimension_reference.element_name
                 and time_dimension_spec.identifier_links == ()
             ):
                 return True
@@ -508,26 +439,26 @@ class MetricFlowQueryParser:
     def _metrics_have_same_time_granularities(self, metric_specs: Sequence[MetricSpec]) -> bool:
         (min_granularity, max_granularity,) = self._time_granularity_solver.local_dimension_granularity_range(
             metric_specs=metric_specs,
-            local_time_dimension_reference=self._primary_time_dimension_reference,
+            local_time_dimension_reference=self._metric_time_dimension_reference,
         )
         return min_granularity == max_granularity
 
-    def _find_smallest_primary_time_dimension_spec_granularity(
+    def _find_smallest_metric_time_dimension_spec_granularity(
         self, time_dimension_specs: Sequence[TimeDimensionSpec]
     ) -> Optional[TimeGranularity]:
-        primary_time_dimension_specs: List[TimeDimensionSpec] = [
+        metric_time_dimension_specs: List[TimeDimensionSpec] = [
             x
             for x in time_dimension_specs
             if (
-                x.element_name == self._primary_time_dimension_reference.element_name
+                x.element_name == self._metric_time_dimension_reference.element_name
                 and x.identifier_links == ()
                 and x.time_granularity
             )
         ]
 
-        primary_time_dimension_specs.sort(key=lambda x: x.time_granularity.to_int())
-        if len(primary_time_dimension_specs) > 0:
-            return primary_time_dimension_specs[0].time_granularity
+        metric_time_dimension_specs.sort(key=lambda x: x.time_granularity.to_int())
+        if len(metric_time_dimension_specs) > 0:
+            return metric_time_dimension_specs[0].time_granularity
         else:
             return None
 
@@ -572,11 +503,9 @@ class MetricFlowQueryParser:
         for qualified_name in qualified_linkable_names:
             structured_name = StructuredLinkableSpecName.from_name(qualified_name)
             element_name = structured_name.element_name
-            identifier_links = tuple(
-                LinklessIdentifierSpec.from_element_name(x) for x in structured_name.identifier_link_names
-            )
+            identifier_links = tuple(IdentifierReference(element_name=x) for x in structured_name.identifier_link_names)
             # Create the spec based on the type of element referenced.
-            if DimensionReference(element_name=element_name) in self._known_time_dimension_element_references:
+            if TimeDimensionReference(element_name=element_name) in self._known_time_dimension_element_references:
                 if structured_name.time_granularity:
                     time_dimension_specs.append(
                         TimeDimensionSpec(
@@ -631,13 +560,7 @@ class MetricFlowQueryParser:
         """Checks that each requested linkable instance can be retrieved for the given metric"""
         invalid_linkable_specs: List[LinkableInstanceSpec] = []
         # TODO: distinguish between dimensions that invalid via typo vs ambiguous join path
-        valid_linkable_specs = self._metric_semantics.element_specs_for_metrics(
-            metric_specs=list(metric_specs),
-            local_only=False,
-            dimensions_only=False,
-            exclude_multi_hop=False,
-            exclude_derived_time_granularities=False,
-        )
+        valid_linkable_specs = self._metric_semantics.element_specs_for_metrics(metric_specs=list(metric_specs))
 
         for dimension_spec in dimension_specs:
             if dimension_spec not in valid_linkable_specs:
@@ -648,7 +571,12 @@ class MetricFlowQueryParser:
                 invalid_linkable_specs.append(identifier_spec)
 
         for time_dimension_spec in time_dimension_specs:
-            if time_dimension_spec not in valid_linkable_specs:
+            if (
+                time_dimension_spec not in valid_linkable_specs
+                # Because the metric time dimension is a virtual dimension that's not in the model, it won't be included
+                # in valid_linkable_specs.
+                and time_dimension_spec.reference != DataSet.metric_time_dimension_reference()
+            ):
                 invalid_linkable_specs.append(time_dimension_spec)
 
         return invalid_linkable_specs
@@ -680,7 +608,7 @@ class MetricFlowQueryParser:
                     )
                 order_by_specs.append(
                     OrderBySpec(
-                        item=MetricSpec(element_name=parsed_name.element_name),
+                        metric_spec=MetricSpec(element_name=parsed_name.element_name),
                         descending=descending,
                     )
                 )
@@ -692,26 +620,24 @@ class MetricFlowQueryParser:
                     )
                 order_by_specs.append(
                     OrderBySpec(
-                        item=DimensionSpec(
+                        dimension_spec=DimensionSpec(
                             element_name=parsed_name.element_name,
                             identifier_links=tuple(
-                                LinklessIdentifierSpec.from_element_name(x) for x in parsed_name.identifier_link_names
+                                IdentifierReference(element_name=x) for x in parsed_name.identifier_link_names
                             ),
                         ),
                         descending=descending,
                     )
                 )
             elif (
-                DimensionReference(element_name=parsed_name.element_name)
+                TimeDimensionReference(element_name=parsed_name.element_name)
                 in self._known_time_dimension_element_references
             ):
-                identifier_links = tuple(
-                    LinklessIdentifierSpec.from_element_name(x) for x in parsed_name.identifier_link_names
-                )
+                identifier_links = tuple(IdentifierReference(element_name=x) for x in parsed_name.identifier_link_names)
                 if parsed_name.time_granularity:
                     order_by_specs.append(
                         OrderBySpec(
-                            item=TimeDimensionSpec(
+                            time_dimension_spec=TimeDimensionSpec(
                                 element_name=parsed_name.element_name,
                                 identifier_links=identifier_links,
                                 time_granularity=parsed_name.time_granularity,
@@ -730,7 +656,7 @@ class MetricFlowQueryParser:
                     if partial_time_dimension_spec in time_dimension_spec_replacements:
                         order_by_specs.append(
                             OrderBySpec(
-                                item=time_dimension_spec_replacements[partial_time_dimension_spec],
+                                time_dimension_spec=time_dimension_spec_replacements[partial_time_dimension_spec],
                                 descending=descending,
                             )
                         )
@@ -749,10 +675,10 @@ class MetricFlowQueryParser:
                     )
                 order_by_specs.append(
                     OrderBySpec(
-                        item=IdentifierSpec(
+                        identifier_spec=IdentifierSpec(
                             element_name=parsed_name.element_name,
                             identifier_links=tuple(
-                                LinklessIdentifierSpec.from_element_name(x) for x in parsed_name.identifier_link_names
+                                IdentifierReference(element_name=x) for x in parsed_name.identifier_link_names
                             ),
                         ),
                         descending=descending,
