@@ -12,14 +12,18 @@ from __future__ import annotations
 import itertools
 from abc import ABC, abstractmethod
 from collections import OrderedDict
-from typing import List, Optional, Sequence, Tuple, TypeVar, Generic
+from dataclasses import dataclass
+from typing import List, Optional, Sequence, Tuple, TypeVar, Generic, Any
+from metricflow.aggregation_properties import AggregationType, AggregationState
 
 from metricflow.column_assoc import ColumnAssociation
 from metricflow.constraints.time_constraint import TimeRangeConstraint
-from metricflow.time.time_granularity import TimeGranularity
-from metricflow.model.objects.utils import ParseableField, FrozenBaseModel
+from metricflow.dataclass_serialization import SerializableDataclass
 from metricflow.naming.linkable_spec_name import StructuredLinkableSpecName
+from metricflow.object_utils import assert_exactly_one_arg_set, hash_strings
+from metricflow.references import DimensionReference, MeasureReference, TimeDimensionReference, IdentifierReference
 from metricflow.sql.sql_bind_parameters import SqlBindParameters
+from metricflow.time.time_granularity import TimeGranularity
 
 
 class ColumnAssociationResolver(ABC):
@@ -52,7 +56,9 @@ class ColumnAssociationResolver(ABC):
         pass
 
     @abstractmethod
-    def resolve_time_dimension_spec(self, time_dimension_spec: TimeDimensionSpec) -> ColumnAssociation:  # noqa: D
+    def resolve_time_dimension_spec(  # noqa: D
+        self, time_dimension_spec: TimeDimensionSpec, aggregation_state: Optional[AggregationState] = None
+    ) -> ColumnAssociation:
         pass
 
     @abstractmethod
@@ -60,27 +66,26 @@ class ColumnAssociationResolver(ABC):
         pass
 
 
-class InstanceSpec(ABC, FrozenBaseModel):
+@dataclass(frozen=True)
+class InstanceSpec(SerializableDataclass):
     """A specification for an instance of a metric definition object.
 
     An instance is different from the definition object in that it correlates to columns in the data flow and can be in
     different states. e.g. a time dimension at a different time granularity.
 
-    Added "type: ignore" as there is an issue with abstract methods in frozen dataclasses:
-    https://github.com/python/mypy/issues/5374
+    This can't be a Protocol as base classes of Protocols need to be Protocols.
     """
 
     """Name of the dimension or identifier in the data source."""
     element_name: str
 
-    @abstractmethod
     def column_associations(self, resolver: ColumnAssociationResolver) -> Tuple[ColumnAssociation, ...]:
         """Figures out what columns in an SQL query that this spec should be associated with given the resolver.
 
         Debating whether this should be an abstract method, or whether it could just live in the different specs to
         allow for different signatures.
         """
-        pass
+        raise NotImplementedError()
 
     @staticmethod
     def merge(*specs: Sequence[InstanceSpec]) -> List[InstanceSpec]:
@@ -91,13 +96,13 @@ class InstanceSpec(ABC, FrozenBaseModel):
         return result
 
     @property
-    @abstractmethod
     def qualified_name(self) -> str:
         """Return the qualified name of this spec. e.g. "user_id__country"."""
-        pass
+        raise NotImplementedError()
 
 
-class LinkableInstanceSpec(InstanceSpec, ABC):
+@dataclass(frozen=True)
+class LinkableInstanceSpec(InstanceSpec):
     """Generally a dimension or identifier that may be specified using identifier links.
 
     For example, user_id__country -> LinkableElementSpec(element_name="country", identifier_links=["user_id"]
@@ -106,22 +111,15 @@ class LinkableInstanceSpec(InstanceSpec, ABC):
     """
 
     """A list representing the join path of identifiers to get to this element."""
-    identifier_links: Tuple[LinklessIdentifierSpec, ...]
+    identifier_links: Tuple[IdentifierReference, ...]
 
-    def __post_init__(self) -> None:  # noqa: D
-        for identifier in self.identifier_links:
-            if len(identifier.identifier_links) > 0:
-                raise RuntimeError("Identifier links may not have their own identifier links")
-
-    @abstractmethod
     def without_first_identifier_link(self) -> LinkableInstanceSpec:
         """e.g. user_id__device_id__platform -> device_id__platform"""
-        pass
+        raise NotImplementedError()
 
-    @abstractmethod
     def without_identifier_links(self) -> LinkableInstanceSpec:  # noqa: D
         """e.g. user_id__device_id__platform -> platform"""
-        pass
+        raise NotImplementedError()
 
     @staticmethod
     def merge_linkable_specs(*specs: Sequence[LinkableInstanceSpec]) -> List[LinkableInstanceSpec]:
@@ -139,7 +137,8 @@ class LinkableInstanceSpec(InstanceSpec, ABC):
         ).qualified_name
 
 
-class IdentifierSpec(LinkableInstanceSpec, ParseableField):  # noqa: D
+@dataclass(frozen=True)
+class IdentifierSpec(LinkableInstanceSpec, SerializableDataclass):  # noqa: D
     def column_associations(self, resolver: ColumnAssociationResolver) -> Tuple[ColumnAssociation, ...]:  # noqa: D
         return resolver.resolve_identifier_spec(self)
 
@@ -151,25 +150,36 @@ class IdentifierSpec(LinkableInstanceSpec, ParseableField):  # noqa: D
         return LinklessIdentifierSpec.from_element_name(self.element_name)
 
     @property
-    def as_linkless_prefix(self) -> Tuple[LinklessIdentifierSpec, ...]:
+    def as_linkless_prefix(self) -> Tuple[IdentifierReference, ...]:
         """Creates tuple of linkless identifiers that could be included in the identifier_links of another spec
 
         eg as a prefix to a DimensionSpec's identifier links to when a join is occurring via this identifier
         """
-        return (LinklessIdentifierSpec.from_element_name(self.element_name),) + self.identifier_links
+        return (IdentifierReference(element_name=self.element_name),) + self.identifier_links
 
     @staticmethod
-    def parse(name: str) -> IdentifierSpec:  # noqa: D
+    def from_name(name: str) -> IdentifierSpec:  # noqa: D
         structured_name = StructuredLinkableSpecName.from_name(name)
         return IdentifierSpec(
-            identifier_links=tuple(
-                LinklessIdentifierSpec.from_element_name(idl) for idl in structured_name.identifier_link_names
-            ),
+            identifier_links=tuple(IdentifierReference(idl) for idl in structured_name.identifier_link_names),
             element_name=structured_name.element_name,
         )
 
+    def __eq__(self, other: Any) -> bool:  # type: ignore[misc] # noqa: D
+        if not isinstance(other, IdentifierSpec):
+            return False
+        return self.element_name == other.element_name and self.identifier_links == other.identifier_links
 
-class LinklessIdentifierSpec(IdentifierSpec):
+    def __hash__(self) -> int:  # noqa: D
+        return hash((self.element_name, self.identifier_links))
+
+    @property
+    def reference(self) -> IdentifierReference:  # noqa: D
+        return IdentifierReference(element_name=self.element_name)
+
+
+@dataclass(frozen=True)
+class LinklessIdentifierSpec(IdentifierSpec, SerializableDataclass):
     """Similar to IdentifierSpec, but requires that it doesn't have identifier links."""
 
     @staticmethod
@@ -180,12 +190,24 @@ class LinklessIdentifierSpec(IdentifierSpec):
         if len(self.identifier_links) > 0:
             raise RuntimeError(f"{self.__class__.__name__} shouldn't have identifier links. Got: {self}")
 
+    def __eq__(self, other: Any) -> bool:  # type: ignore[misc] # noqa: D
+        if not isinstance(other, IdentifierSpec):
+            return False
+        return self.element_name == other.element_name and self.identifier_links == other.identifier_links
 
-IdentifierSpec.update_forward_refs()
-LinklessIdentifierSpec.update_forward_refs()
+    def __hash__(self) -> int:  # noqa: D
+        return hash((self.element_name, self.identifier_links))
+
+    @staticmethod
+    def from_reference(identifier_reference: IdentifierReference) -> LinklessIdentifierSpec:  # noqa: D
+        return LinklessIdentifierSpec(element_name=identifier_reference.element_name, identifier_links=())
 
 
-class DimensionSpec(LinkableInstanceSpec, ParseableField):  # noqa: D
+@dataclass(frozen=True)
+class DimensionSpec(LinkableInstanceSpec, SerializableDataclass):  # noqa: D
+    element_name: str
+    identifier_links: Tuple[IdentifierReference, ...]
+
     def column_associations(self, resolver: ColumnAssociationResolver) -> Tuple[ColumnAssociation, ...]:  # noqa: D
         return (resolver.resolve_dimension_spec(self),)
 
@@ -201,23 +223,24 @@ class DimensionSpec(LinkableInstanceSpec, ParseableField):  # noqa: D
         return DimensionSpec(element_name=spec.element_name, identifier_links=spec.identifier_links)
 
     @staticmethod
-    def parse(name: str) -> DimensionSpec:
+    def from_name(name: str) -> DimensionSpec:
         """Construct from a name e.g. listing__ds__month."""
         parsed_name = StructuredLinkableSpecName.from_name(name)
         return DimensionSpec(
-            identifier_links=tuple(
-                [LinklessIdentifierSpec.from_element_name(idl) for idl in parsed_name.identifier_link_names]
-            ),
+            identifier_links=tuple([IdentifierReference(idl) for idl in parsed_name.identifier_link_names]),
             element_name=parsed_name.element_name,
         )
 
+    @property
+    def reference(self) -> DimensionReference:  # noqa: D
+        return DimensionReference(element_name=self.element_name)
 
-DimensionSpec.update_forward_refs()
 
 DEFAULT_TIME_GRANULARITY = TimeGranularity.DAY
 
 
-class TimeDimensionSpec(DimensionSpec, ParseableField):  # noqa: D
+@dataclass(frozen=True)
+class TimeDimensionSpec(DimensionSpec):  # noqa: D
     time_granularity: TimeGranularity = DEFAULT_TIME_GRANULARITY
 
     def column_associations(self, resolver: ColumnAssociationResolver) -> Tuple[ColumnAssociation, ...]:  # noqa: D
@@ -235,12 +258,10 @@ class TimeDimensionSpec(DimensionSpec, ParseableField):  # noqa: D
         return LinklessIdentifierSpec.from_element_name(self.element_name)
 
     @staticmethod
-    def parse(name: str) -> TimeDimensionSpec:  # noqa: D
+    def from_name(name: str) -> TimeDimensionSpec:  # noqa: D
         structured_name = StructuredLinkableSpecName.from_name(name)
         return TimeDimensionSpec(
-            identifier_links=tuple(
-                LinklessIdentifierSpec.from_element_name(idl) for idl in structured_name.identifier_link_names
-            ),
+            identifier_links=tuple(IdentifierReference(idl) for idl in structured_name.identifier_link_names),
             element_name=structured_name.element_name,
             time_granularity=structured_name.time_granularity or DEFAULT_TIME_GRANULARITY,
         )
@@ -249,8 +270,9 @@ class TimeDimensionSpec(DimensionSpec, ParseableField):  # noqa: D
     def reference(self) -> TimeDimensionReference:  # noqa: D
         return TimeDimensionReference(element_name=self.element_name)
 
-    def matches_reference(self, time_dimension_reference: TimeDimensionReference) -> bool:  # noqa: D
-        return self.element_name == time_dimension_reference.element_name
+    @property
+    def dimension_reference(self) -> DimensionReference:  # noqa: D
+        return DimensionReference(element_name=self.element_name)
 
     @property
     def qualified_name(self) -> str:  # noqa: D
@@ -261,49 +283,59 @@ class TimeDimensionSpec(DimensionSpec, ParseableField):  # noqa: D
         ).qualified_name
 
 
-class ElementReference(FrozenBaseModel, ParseableField):
-    """Used when we need to refer to a dimension, measure, identifier, but other attributes are unknown."""
+@dataclass(frozen=True)
+class NonAdditiveDimensionSpec(SerializableDataclass):
+    """Spec representing non-additive dimension parameters for use within a MeasureSpec
 
-    @staticmethod
-    def parse(name: str) -> ElementReference:  # noqa: D
-        return ElementReference(element_name=name)
+    This is sourced from the NonAdditiveDimensionParameters model object, which provides the parsed parameter set,
+    while the spec contains the information needed for dataflow plan operations
+    """
 
+    name: str
+    window_choice: AggregationType
+    window_groupings: Tuple[str, ...] = ()
+
+    @property
+    def bucket_hash(self) -> str:
+        """Returns the hash value used for grouping equivalent params."""
+        values = [self.window_choice.name, self.name]
+        values.extend(sorted(self.window_groupings))
+        return hash_strings(values)
+
+    @property
+    def linkable_specs(self) -> LinkableSpecSet:  # noqa: D
+        return LinkableSpecSet(
+            dimension_specs=(),
+            time_dimension_specs=(TimeDimensionSpec.from_name(self.name),),
+            identifier_specs=tuple(
+                LinklessIdentifierSpec.from_element_name(identifier_name) for identifier_name in self.window_groupings
+            ),
+        )
+
+    def __eq__(self, other: Any) -> bool:  # type: ignore[misc] # noqa: D
+        if not isinstance(other, NonAdditiveDimensionSpec):
+            return False
+        return self.bucket_hash == other.bucket_hash
+
+
+@dataclass(frozen=True)
+class MeasureSpec(InstanceSpec):  # noqa: D
     element_name: str
+    non_additive_dimension_spec: Optional[NonAdditiveDimensionSpec] = None
 
-
-class LinkableElementReference(ElementReference):
-    """Used when we need to refer to a dimension or identifier, but other attributes are unknown."""
-
-    pass
-
-
-class MeasureReference(ElementReference):
-    """Used when we need to refer to a measure (separate from LinkableElementReference because measures aren't linkable"""
-
-    pass
-
-
-class DimensionReference(LinkableElementReference):  # noqa: D
-    pass
-
-
-class IdentifierReference(LinkableElementReference):  # noqa: D
-    pass
-
-
-class TimeDimensionReference(DimensionReference):  # noqa: D
-    pass
-
-
-class MeasureSpec(InstanceSpec, ParseableField):  # noqa: D
     def column_associations(self, resolver: ColumnAssociationResolver) -> Tuple[ColumnAssociation, ...]:  # noqa: D
         return (resolver.resolve_measure_spec(self),)
 
     @staticmethod
-    def parse(name: str) -> MeasureSpec:
+    def from_name(name: str) -> MeasureSpec:
         """Construct from a name e.g. listing__ds__month."""
         return MeasureSpec(element_name=name)
 
+    @staticmethod
+    def from_reference(reference: MeasureReference) -> MeasureSpec:
+        """Initialize from a measure reference instance"""
+        return MeasureSpec(element_name=reference.element_name)
+
     @property
     def qualified_name(self) -> str:  # noqa: D
         return self.element_name
@@ -313,8 +345,10 @@ class MeasureSpec(InstanceSpec, ParseableField):  # noqa: D
         return MeasureReference(element_name=self.element_name)
 
 
+@dataclass(frozen=True)
 class MetricSpec(InstanceSpec):  # noqa: D
     # Time-over-time could go here
+    element_name: str
 
     def column_associations(self, resolver: ColumnAssociationResolver) -> Tuple[ColumnAssociation, ...]:  # noqa: D
         return (resolver.resolve_metric_spec(self),)
@@ -324,17 +358,70 @@ class MetricSpec(InstanceSpec):  # noqa: D
         return self.element_name
 
 
-class OrderBySpec(FrozenBaseModel):  # noqa: D
-    item: InstanceSpec
+@dataclass(frozen=True)
+class MetricInputMeasureSpec(SerializableDataclass):
+    """The spec for a measure defined as a metric input.
+
+    This is necessary because the MeasureSpec is used as a key linking the measures used in the query
+    to the measures defined in the data sources. Adding metric-specific information, like constraints,
+    causes lookups connecting query -> data source to fail in strange ways. This spec, then, provides
+    both the key (in the form of a MeasureSpec) along with whatever measure-specific attributes
+    a user might specify in a metric definition or query accessing the metric itself.
+
+    Note - when specifying a metric comprised of two input instances of the same measure, at least one
+    must have a distinct alias, otherwise SQL exceptions may occur. This should be enforced via validation.
+    """
+
+    measure_spec: MeasureSpec
+    constraint: Optional[SpecWhereClauseConstraint] = None
+    alias: Optional[str] = None
+
+    @property
+    def post_aggregation_spec(self) -> MeasureSpec:
+        """Return a MeasureSpec instance representing the post-aggregation spec state for the underlying measure"""
+        if self.alias:
+            return MeasureSpec(
+                element_name=self.alias,
+                non_additive_dimension_spec=self.measure_spec.non_additive_dimension_spec,
+            )
+        else:
+            return self.measure_spec
+
+
+@dataclass(frozen=True)
+class OrderBySpec(SerializableDataclass):  # noqa: D
+
     descending: bool
+    metric_spec: Optional[MetricSpec] = None
+    dimension_spec: Optional[DimensionSpec] = None
+    time_dimension_spec: Optional[TimeDimensionSpec] = None
+    identifier_spec: Optional[IdentifierSpec] = None
+
+    def __post_init__(self) -> None:  # noqa: D
+        assert_exactly_one_arg_set(
+            metric_spec=self.metric_spec,
+            dimension_spec=self.dimension_spec,
+            time_dimension_spec=self.time_dimension_spec,
+            identifier_spec=self.identifier_spec,
+        )
+
+    @property
+    def item(self) -> InstanceSpec:  # noqa: D
+        result: Optional[InstanceSpec] = (
+            self.metric_spec or self.dimension_spec or self.time_dimension_spec or self.identifier_spec
+        )
+        assert result
+        return result
 
 
-class FilterSpec(FrozenBaseModel):  # noqa: D
+@dataclass(frozen=True)
+class FilterSpec(SerializableDataclass):  # noqa: D
     expr: str
     elements: Tuple[InstanceSpec, ...]
 
 
-class OutputColumnNameOverride(FrozenBaseModel):
+@dataclass(frozen=True)
+class OutputColumnNameOverride(SerializableDataclass):
     """Describes how we should name the output column for a time dimension instead of the default.
 
     Note: This is used temporarily to maintain compatibility with the old framework.
@@ -344,7 +431,8 @@ class OutputColumnNameOverride(FrozenBaseModel):
     output_column_name: str
 
 
-class LinkableSpecSet(FrozenBaseModel):
+@dataclass(frozen=True)
+class LinkableSpecSet(SerializableDataclass):
     """Groups linkable specs."""
 
     dimension_specs: Tuple[DimensionSpec, ...] = ()
@@ -384,7 +472,8 @@ class LinkableSpecSet(FrozenBaseModel):
         return set(self.as_tuple).issubset(set(other_set.as_tuple))
 
 
-class SpecWhereClauseConstraint(FrozenBaseModel):
+@dataclass(frozen=True)
+class SpecWhereClauseConstraint(SerializableDataclass):
     """Similar to a WhereClauseConstraint, but with specs instead of strings"""
 
     # e.g. "listing__capacity_latest > 4"
@@ -415,7 +504,8 @@ class SpecWhereClauseConstraint(FrozenBaseModel):
         )
 
 
-class MetricFlowQuerySpec(FrozenBaseModel):
+@dataclass(frozen=True)
+class MetricFlowQuerySpec(SerializableDataclass):
     """Specs needed for running a query."""
 
     metric_specs: Tuple[MetricSpec, ...] = ()
@@ -448,7 +538,8 @@ class InstanceSpecSetTransform(Generic[TransformOutputT], ABC):
         pass
 
 
-class InstanceSpecSet(FrozenBaseModel):
+@dataclass(frozen=True)
+class InstanceSpecSet(SerializableDataclass):
     """Consolidates all specs used in an instance set"""
 
     metric_specs: Tuple[MetricSpec, ...] = ()
@@ -490,7 +581,8 @@ class InstanceSpecSet(FrozenBaseModel):
         return transform_function.transform(self)
 
 
-class PartitionSpecSet(FrozenBaseModel):
+@dataclass(frozen=True)
+class PartitionSpecSet(SerializableDataclass):
     """Grouping of the linkable specs."""
 
     dimension_specs: Tuple[DimensionSpec, ...] = ()
