@@ -1,13 +1,10 @@
-from typing import Tuple
 import pytest
 import copy
 
 from metricflow.model.model_validator import ModelValidator
-from metricflow.model.objects.data_source import DataSource
-from metricflow.model.objects.elements.dimension import Dimension, DimensionType
 from metricflow.model.validations.validator_helpers import ModelValidationException
 from metricflow.model.objects.user_configured_model import UserConfiguredModel
-from metricflow.model.validations.unique_valid_name import UniqueAndValidNameRule
+from metricflow.model.validations.unique_valid_name import MetricFlowReservedKeywords, UniqueAndValidNameRule
 from metricflow.object_utils import flatten_nested_sequence
 from metricflow.test.test_utils import find_data_source_with
 
@@ -16,10 +13,6 @@ def copied_model(simple_model__pre_transforms: UserConfiguredModel) -> UserConfi
     return copy.deepcopy(simple_model__pre_transforms)
 
 
-def _categorical_dimensions(data_source: DataSource) -> Tuple[Dimension, ...]:
-    return tuple(dim for dim in data_source.dimensions if dim.type == DimensionType.CATEGORICAL)
-
-
 """
     Top Level Tests
     Top level elements include
@@ -42,7 +35,7 @@ def test_duplicate_data_source_name(simple_model__pre_transforms: UserConfigured
         ModelValidationException,
         match=rf"Can't use name `{duplicated_data_source.name}` for a data source when it was already used for a data source",
     ):
-        ModelValidator.checked_validations(model)
+        ModelValidator().checked_validations(model)
 
 
 def test_duplicate_metric_name(simple_model__pre_transforms: UserConfiguredModel) -> None:  # noqa:D
@@ -53,7 +46,7 @@ def test_duplicate_metric_name(simple_model__pre_transforms: UserConfiguredModel
         ModelValidationException,
         match=rf"Can't use name `{duplicated_metric.name}` for a metric when it was already used for a metric",
     ):
-        ModelValidator.checked_validations(model)
+        ModelValidator().checked_validations(model)
 
 
 def test_duplicate_materalization_name(simple_model__pre_transforms: UserConfiguredModel) -> None:  # noqa:D
@@ -64,7 +57,7 @@ def test_duplicate_materalization_name(simple_model__pre_transforms: UserConfigu
         ModelValidationException,
         match=rf"Can't use name `{duplicated_materialization.name}` for a materialization when it was already used for a materialization",
     ):
-        ModelValidator.checked_validations(model)
+        ModelValidator().checked_validations(model)
 
 
 def test_top_level_metric_can_have_same_name_as_any_other_top_level_item(
@@ -78,8 +71,8 @@ def test_top_level_metric_can_have_same_name_as_any_other_top_level_item(
     model_data_source.data_sources[0].name = metric_name
     model_materialization.materializations[0].name = model_data_source.metrics[0].name
 
-    ModelValidator.checked_validations(model_data_source)
-    ModelValidator.checked_validations(model_materialization)
+    ModelValidator().checked_validations(model_data_source)
+    ModelValidator().checked_validations(model_materialization)
 
 
 def test_top_level_elements_cant_share_names_except_with_metrics(
@@ -93,7 +86,7 @@ def test_top_level_elements_cant_share_names_except_with_metrics(
         ModelValidationException,
         match=rf"Can't use name `{data_source_name}` for a materialization when it was already used for a data source",
     ):
-        ModelValidator.checked_validations(model_ds_and_mat)
+        ModelValidator().checked_validations(model_ds_and_mat)
 
 
 """
@@ -108,53 +101,6 @@ def test_top_level_elements_cant_share_names_except_with_metrics(
 """
 
 
-def test_cross_element_names(simple_model__pre_transforms: UserConfiguredModel) -> None:  # noqa:D
-    model = copied_model(simple_model__pre_transforms)
-
-    # ensure we have a usable data source for the test
-    usable_ds, usable_ds_index = find_data_source_with(
-        model,
-        lambda data_source: len(data_source.measures) > 0
-        and len(data_source.identifiers) > 0
-        and len(_categorical_dimensions(data_source=data_source)) > 0,
-    )
-
-    measure_reference = usable_ds.measures[0].name
-    # If the matching dimension is a time dimension we can accidentally create two primary time dimensions, and then
-    # the validation will throw a different error and fail the test
-    dimension_reference = _categorical_dimensions(data_source=usable_ds)[0].name
-
-    ds_measure_x_dimension = copy.deepcopy(usable_ds)
-    ds_measure_x_identifier = copy.deepcopy(usable_ds)
-    ds_dimension_x_identifier = copy.deepcopy(usable_ds)
-
-    # We update the matching categorical dimension by reference for convenience
-    ds_measure_x_dimension.get_dimension(dimension_reference).name = measure_reference
-    ds_measure_x_identifier.identifiers[1].name = measure_reference
-    ds_dimension_x_identifier.identifiers[1].name = dimension_reference
-
-    model.data_sources[usable_ds_index] = ds_measure_x_dimension
-    with pytest.raises(
-        ModelValidationException,
-        match=rf"can't use name `{measure_reference.element_name}` for a dimension when it was already used for a measure",
-    ):
-        ModelValidator.checked_validations(model)
-
-    model.data_sources[usable_ds_index] = ds_measure_x_identifier
-    with pytest.raises(
-        ModelValidationException,
-        match=rf"can't use name `{measure_reference.element_name}` for a identifier when it was already used for a measure",
-    ):
-        ModelValidator.checked_validations(model)
-
-    model.data_sources[usable_ds_index] = ds_dimension_x_identifier
-    with pytest.raises(
-        ModelValidationException,
-        match=rf"can't use name `{dimension_reference.element_name}` for a dimension when it was already used for a identifier",
-    ):
-        ModelValidator.checked_validations(model)
-
-
 def test_duplicate_measure_name(simple_model__pre_transforms: UserConfiguredModel) -> None:  # noqa:D
     model = copied_model(simple_model__pre_transforms)
 
@@ -167,9 +113,9 @@ def test_duplicate_measure_name(simple_model__pre_transforms: UserConfiguredMode
 
     with pytest.raises(
         ModelValidationException,
-        match=rf"can't use name `{duplicated_measure.name.element_name}` for a measure when it was already used for a measure",
+        match=rf"can't use name `{duplicated_measure.reference.element_name}` for a measure when it was already used for a measure",
     ):
-        ModelValidator.checked_validations(model)
+        ModelValidator().checked_validations(model)
 
 
 def test_duplicate_dimension_name(simple_model__pre_transforms: UserConfiguredModel) -> None:  # noqa:D
@@ -184,10 +130,10 @@ def test_duplicate_dimension_name(simple_model__pre_transforms: UserConfiguredMo
 
     with pytest.raises(
         ModelValidationException,
-        match=rf"can't use name `{duplicated_dimension.name.element_name}` for a "
+        match=rf"can't use name `{duplicated_dimension.reference.element_name}` for a "
         rf"dimension when it was already used for a dimension",
     ):
-        ModelValidator.checked_validations(model)
+        ModelValidator().checked_validations(model)
 
 
 def test_duplicate_identifier_name(simple_model__pre_transforms: UserConfiguredModel) -> None:  # noqa:D
@@ -202,9 +148,9 @@ def test_duplicate_identifier_name(simple_model__pre_transforms: UserConfiguredM
 
     with pytest.raises(
         ModelValidationException,
-        match=rf"can't use name `{duplicated_identifier.name.element_name}` for a identifier when it was already used for a identifier",
+        match=rf"can't use name `{duplicated_identifier.reference.element_name}` for a identifier when it was already used for a identifier",
     ):
-        ModelValidator.checked_validations(model)
+        ModelValidator().checked_validations(model)
 
 
 """
@@ -237,3 +183,16 @@ def test_invalid_names() -> None:  # noqa:D
     assert UniqueAndValidNameRule.check_valid_name("month") != []
     assert UniqueAndValidNameRule.check_valid_name("quarter") != []
     assert UniqueAndValidNameRule.check_valid_name("year") != []
+
+
+def test_reserved_name() -> None:  # noqa: D
+    reserved_keyword = MetricFlowReservedKeywords.METRIC_TIME
+    reserved_reason = MetricFlowReservedKeywords.get_reserved_reason(reserved_keyword)
+    issues = UniqueAndValidNameRule.check_valid_name(reserved_keyword.value.lower())
+    match = False
+    for issue in issues:
+        if issue.message.find(reserved_reason) != -1:
+            match = True
+    assert (
+        match
+    ), f"Did not find reason: '{reserved_reason}' in issues: {issues} for name: '{reserved_keyword.value.lower()}'"
