@@ -2,23 +2,23 @@
 -- Compute Metrics via Expressions
 -- Order By ['ds', 'bookings']
 SELECT
-  SUM(bookings) AS bookings
+  ds
   , is_instant
-  , ds
+  , SUM(bookings) AS bookings
 FROM (
   -- Read Elements From Data Source 'bookings_source'
   -- Pass Only Elements:
   --   ['bookings', 'is_instant', 'ds']
   SELECT
-    1 AS bookings
+    ds
     , is_instant
-    , ds
+    , 1 AS bookings
   FROM (
     -- User Defined SQL Query
     SELECT * FROM ***************************.fct_bookings
-  ) bookings_source_src_10000
+  ) bookings_source_src_10001
 ) subq_5
 GROUP BY
-  is_instant
-  , ds
+  ds
+  , is_instant
 ORDER BY ds, bookings DESC
