@@ -7,6 +7,7 @@ SELECT
   , bookings_source_src_10000.booking_value AS min_booking_value
   , bookings_source_src_10000.guest_id AS bookers
   , bookings_source_src_10000.booking_value AS average_booking_value
+  , bookings_source_src_10000.booking_value AS booking_payments
   , bookings_source_src_10000.is_instant
   , bookings_source_src_10000.ds
   , '__DATE_TRUNC_NOT_SUPPORTED__' AS ds__week
@@ -18,6 +19,11 @@ SELECT
   , '__DATE_TRUNC_NOT_SUPPORTED__' AS ds_partitioned__month
   , '__DATE_TRUNC_NOT_SUPPORTED__' AS ds_partitioned__quarter
   , '__DATE_TRUNC_NOT_SUPPORTED__' AS ds_partitioned__year
+  , bookings_source_src_10000.booking_paid_at
+  , '__DATE_TRUNC_NOT_SUPPORTED__' AS booking_paid_at__week
+  , '__DATE_TRUNC_NOT_SUPPORTED__' AS booking_paid_at__month
+  , '__DATE_TRUNC_NOT_SUPPORTED__' AS booking_paid_at__quarter
+  , '__DATE_TRUNC_NOT_SUPPORTED__' AS booking_paid_at__year
   , bookings_source_src_10000.is_instant AS create_a_cycle_in_the_join_graph__is_instant
   , bookings_source_src_10000.ds AS create_a_cycle_in_the_join_graph__ds
   , '__DATE_TRUNC_NOT_SUPPORTED__' AS create_a_cycle_in_the_join_graph__ds__week
@@ -29,6 +35,11 @@ SELECT
   , '__DATE_TRUNC_NOT_SUPPORTED__' AS create_a_cycle_in_the_join_graph__ds_partitioned__month
   , '__DATE_TRUNC_NOT_SUPPORTED__' AS create_a_cycle_in_the_join_graph__ds_partitioned__quarter
   , '__DATE_TRUNC_NOT_SUPPORTED__' AS create_a_cycle_in_the_join_graph__ds_partitioned__year
+  , bookings_source_src_10000.booking_paid_at AS create_a_cycle_in_the_join_graph__booking_paid_at
+  , '__DATE_TRUNC_NOT_SUPPORTED__' AS create_a_cycle_in_the_join_graph__booking_paid_at__week
+  , '__DATE_TRUNC_NOT_SUPPORTED__' AS create_a_cycle_in_the_join_graph__booking_paid_at__month
+  , '__DATE_TRUNC_NOT_SUPPORTED__' AS create_a_cycle_in_the_join_graph__booking_paid_at__quarter
+  , '__DATE_TRUNC_NOT_SUPPORTED__' AS create_a_cycle_in_the_join_graph__booking_paid_at__year
   , bookings_source_src_10000.listing_id AS listing
   , bookings_source_src_10000.guest_id AS guest
   , bookings_source_src_10000.host_id AS host
