@@ -1,7 +1,7 @@
 -- query
 SELECT
   SUM(bookings_src.bookings) AS bookings
-  , listings_src.listing__country_latest AS ds
+  , listings_src.country_latest AS listing__country_latest
   , bookings_src.ds AS ds
 FROM (
   -- bookings_src
